package com.chinaitop.depot.agent.archive.controller;

import javax.annotation.Resource;

import com.chinaitop.depot.agent.archive.model.StorageArchive;
import com.chinaitop.depot.agent.archive.model.StorageArchiveExample;
import com.chinaitop.depot.agent.archive.service.ArchiveService;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.utils.ParameterUtil;
import com.github.pagehelper.PageHelper;

import java.util.*;
import java.util.concurrent.Future;

@RestController
@RequestMapping(value = "/archiveData")
@Api(description = "数据归档管理")
public class ArchiveController {

    @Resource
    private ArchiveService archiveService;

    /**
     * 查询仓储数据归档记录
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getArchiveList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="仓储数据归档", notes = "查询仓储数据归档记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "storehouseId", value = "仓房id", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "归档开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "归档结束时间", paramType = "query"),
            @ApiImplicitParam(name = "archiveState", value = "归档状态", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织Id", paramType = "query"),
            @ApiImplicitParam(name = "userId", value = "用户Id", paramType = "query")
    })
    public PageInfo<StorageArchive> getArchiveList(Integer pageNum, Integer pageSize, String storehouseId, String startTime, String endTime, String archiveState, String orgId, Integer userId){
        PageHelper.startPage(pageNum, pageSize);
        List<StorageArchive> list = archiveService.getArchiveList(storehouseId,startTime,endTime,archiveState,orgId,userId);
        PageInfo<StorageArchive> pageInfo = new PageInfo<StorageArchive>(list);
        return pageInfo;
    }

    /**
     * 仓储归档新增列表
     * @param userId
     * @param orgId
     * @return
     */
    @RequestMapping(value = "/getArchiveHouseList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="仓房归档列表", notes = "仓储归档新增列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "当前登录人Id", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织Id", paramType = "query")
    })
    public List<Map<String,Object>> getArchiveHouseList(String userId, String orgId){
        List<Map<String,Object>> list = archiveService.getArchiveHouseList(userId, orgId);
        return list;
    }

    /**
     * 提交相关归档仓房
     *
     * @param houseId 仓房数组
     * @param userInfo 登录数据对象
     * @return
     */
    @RequestMapping(value = "/saveArchiveHouse", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="要归档仓房", notes = "提交相关归档仓房")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "houseId", value = "仓房数组", paramType = "form"),
            @ApiImplicitParam(name = "houseCode", value = "仓房Code数组", paramType = "form"),
            @ApiImplicitParam(name = "userInfo", value = "登录数据对象", paramType = "form"),
            @ApiImplicitParam(name = "archiveState", value = "提交后审批状态", paramType = "form")
    })
    public Map<String, Object> saveArchiveHouse(String houseId, String houseCode, String userInfo, String archiveState) {
        Map<String, Object> map = new HashMap<String, Object>();

        try {
            String archiveIds = archiveService.saveArchiveHouse(houseId, houseCode, userInfo, archiveState);
            map.put("status", "success");
            map.put("archiveIds", archiveIds.substring(0,archiveIds.length()-1));
        } catch (Exception e) {
            e.printStackTrace();
            map.put("status", "error");
            map.put("msg", "归档失败！");
        }
        return map;
    }

    /**
     * 仓储归档审批列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getArchiveApprovalList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="归档审批列表", notes = "仓储归档审批列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "storehouseId", value = "仓房id", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "归档开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "归档结束时间", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织Id", paramType = "query"),
            @ApiImplicitParam(name = "userId", value = "用户Id", paramType = "query")
    })
    public PageInfo<StorageArchive> getArchiveApprovalList(Integer pageNum, Integer pageSize, String storehouseId, String startTime, String endTime, String orgId, Integer userId){
        PageHelper.startPage(pageNum, pageSize);
        List<StorageArchive> list = archiveService.getArchiveApprovalList(storehouseId,startTime,endTime,orgId,userId);
        PageInfo<StorageArchive> pageInfo = new PageInfo<StorageArchive>(list);
        return pageInfo;
    }

    /**
     * 根据id修改审批状态
     * @param archiveId
     * @param archiveState
     * @return
     */
    @RequestMapping(value="/updateArchiveState", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="修改审批状态", notes = "根据id修改审批状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "archiveId", value = "归档id组", paramType = "query"),
            @ApiImplicitParam(name = "archiveState", value = "审批状态", paramType = "query")
    })
    public Map<String,Object> updateArchiveState(String archiveId, String archiveState){
        Map<String, Object> modelMap = new HashMap<String, Object>();
        try {
            archiveService.updateArchiveState(archiveId, archiveState);
            modelMap.put("status", "success");
        }catch (Exception e){
            System.out.println(e);
            modelMap.put("status", "error");
        }
        return modelMap;
    }

    /**
     * 根据id删除归档数据
     * @param archiveId
     * @return
     */
    @RequestMapping(value = "/removeDetail", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="删除归档数据", notes = "根据id删除归档数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "archiveId", value = "归档id", paramType = "query")
    })
    public Map<String, Object> removeDetail(String archiveId){
        Map<String, Object> modelMap = new HashMap<>();
        if (ParameterUtil.isnotnull(archiveId)) {
            archiveService.removeDetail(archiveId);

            modelMap.put("status", "success");
        }
        return modelMap;
    }

    /**
     * 仓储数据查看列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getArchiveViewList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="仓储数据查看", notes = "仓储数据查看列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "storehouseId", value = "仓房id", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "归档开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "归档结束时间", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public PageInfo<StorageArchive> getArchiveViewList(Integer pageNum, Integer pageSize, String storehouseId,
          String startTime, String endTime, String orgId){
        PageHelper.startPage(pageNum, pageSize);
        List<StorageArchive> list = archiveService.getArchiveViewList(storehouseId,startTime,endTime,orgId);
        PageInfo<StorageArchive> pageInfo = new PageInfo<StorageArchive>(list);
        return pageInfo;
    }

    @RequestMapping(value = "/getNewArchive", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="归档仓房", notes = "默认加载的归档仓房数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织Id", paramType = "query")
    })
    public Map<String, Object> getNewArchive(String orgId){
        Map<String, Object> modelMap = new HashMap<>();
        List<StorageArchive> list = archiveService.getArchiveViewList("","","",orgId);
        if (list.size() != 0) {
            modelMap.put("storehouseId", list.get(0).getStorehouseId());
            modelMap.put("storehouseCode", list.get(0).getStorehouseCode());
            modelMap.put("createTime", list.get(0).getCreateTime());
        }else{
            modelMap.put("storehouseId", "");
            modelMap.put("storehouseCode", "");
            modelMap.put("createTime", "");
        }

        return modelMap;
    }

    /**
     * 归档仓房下拉
     *
     * @param orgId 单位ID
     * @return
     */
    @RequestMapping(value = "/getArchiveDataList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="归档仓房下拉", notes = "归档仓房下拉")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query")
    })
    public Map<Integer, Object> getArchiveDataList(String orgId) throws Exception {
        Map<Integer, Object> map = null;
        map = archiveService.getArchiveDataList(orgId);
        return map;
    }
}