package com.chinaitop.depot.agent.archive.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.agent.archive.mapper.StorageArchiveMapper;
import com.chinaitop.depot.agent.archive.model.StorageArchive;
import com.chinaitop.depot.agent.archive.model.StorageArchiveExample;
import com.chinaitop.depot.agent.archive.service.ArchiveService;
import com.chinaitop.utils.ParameterUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Service
public class ArchiveServiceImpl implements ArchiveService {

    @Resource
    private StorageArchiveMapper StorageArchiveMapper;

    /**
     * 查询仓储处作业记录
     * * @param map
     * @return
     */
    @Override
    public List<StorageArchive> getArchiveList(String storehouseId, String startTime, String endTime, String archiveState, String orgId, Integer userId) {
        StorageArchiveExample example = new StorageArchiveExample();
        StorageArchiveExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(orgId);
        criteria.andCreaterIdEqualTo(userId);
        if (ParameterUtil.isnotnull(storehouseId)) {
            criteria.andStorehouseIdEqualTo(storehouseId);
        }
        if (ParameterUtil.isnotnull(startTime) && ParameterUtil.isnotnull(endTime)) {
            criteria.andCreateTimeBetween(ParameterUtil.string2datetime(startTime + " 00:00:00"),ParameterUtil.string2datetime(endTime + " 23:59:59"));
        }
        if (ParameterUtil.isnotnull(archiveState)) {
            if(ParameterUtil.isequal(3,archiveState)){
                List<String> stateList = new ArrayList<String>();
                stateList.add("3");
                stateList.add("4");
                criteria.andArchiveStateIn(stateList);
            }else{
                criteria.andArchiveStateEqualTo(archiveState);
            }
        }
        example.setOrderByClause("updatetime desc");
        List<StorageArchive> storageArchives = StorageArchiveMapper.selectByExample(example);
        //赋值以便重新提交时候用
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("userId", userId);
        map.put("orgId", orgId);

        //先根据userId查询是否是保管员
        List<Map<String,Object>> roleData = StorageArchiveMapper.getRole(map);
        if(roleData.size() != 0){
            for (StorageArchive archiveData:storageArchives) {
                archiveData.setResult(roleData.get(0).get("rolenames").toString());
            }
        }
        return storageArchives;
    }

    @Override
    public List<Map<String,Object>> getArchiveHouseList(String userId, String orgId) {
        Map<String, Object> map = new HashMap<String, Object>();
        if(ParameterUtil.isnotnull(userId)){
            map.put("userId", userId);
        }

        if(ParameterUtil.isnotnull(orgId)){
            map.put("orgId", orgId);
        }

        //先根据userId查询是否是保管员
        List<Map<String,Object>> roleData = new ArrayList<Map<String,Object>>();
        List<Map<String,Object>> houseData = new ArrayList<Map<String,Object>>();
        roleData = StorageArchiveMapper.getRole(map);

        for (Map<String,Object> role : roleData) {
            //判断如果不是保管员则是判断是否是仓储部经理
            if(ParameterUtil.isequal("保管员",role.get("rolenames"))){
                map.put("roleName","保管员");
                houseData = StorageArchiveMapper.getBGYhouse(map);
                return houseData;
            }else if(ParameterUtil.isequal("仓储部经理", role.get("rolenames"))){
                map.put("roleName","仓储部经理");
                houseData = StorageArchiveMapper.getCCBJLhouse(map);
                return houseData;
            }
        }
        houseData.add(0,map);

        return houseData;
    }

    @Override
    public List<StorageArchive> getArchiveApprovalList(String storehouseId, String startTime, String endTime, String orgId, Integer userId) {
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("userId", userId);
        map.put("orgId", orgId);
        if (ParameterUtil.isnotnull(storehouseId)) {
            map.put("storehouseId", storehouseId);
        }
        if (ParameterUtil.isnotnull(startTime) && ParameterUtil.isnotnull(endTime)) {
            map.put("startTime", ParameterUtil.string2datetime(startTime + " 00:00:00"));
            map.put("endTime", ParameterUtil.string2datetime(endTime + " 23:59:59"));
        }

        //先根据userId查询是否是保管员
        List<StorageArchive> roleData = StorageArchiveMapper.getArchiveApprovalList(map);
        return roleData;
    }

    @Override
    public int removeDetail(String archiveId) {
        return StorageArchiveMapper.deleteByPrimaryKey(archiveId);
    }

    @Override
    public int updateArchiveState(String archiveIds, String archiveState) {
        //根据归档id查询要修改的状态数据
        StorageArchiveExample example = new StorageArchiveExample();
        StorageArchiveExample.Criteria criteria = example.createCriteria();
        criteria.andIdEqualTo(archiveIds);
        //添加要修改的状态
        StorageArchive storageArchive = new StorageArchive();
        storageArchive.setArchiveState(archiveState);
        return StorageArchiveMapper.updateByExampleSelective(storageArchive,example);
    }

    @Override
    public String saveArchiveHouse(String houseId, String houseCode, String userInfo, String archiveState) {
        JSONObject user = null;
        String orgId = null;
        Integer userId = null;
        String userName = null;
        if (null != userInfo && !"".equals(userInfo)) {
            user = JSONObject.parseObject(userInfo);
            orgId = user.get("orgId") == null?"":user.get("orgId").toString();
            userId = Integer.parseInt(user.get("userId") == null?"":user.get("userId").toString());
            userName = user.get("realName") == null?"":user.get("realName").toString();
        }

        String[]checkArray = null;
        String[]checkCodeArray = null;
        if (houseId.length() > 2) {
            String ids = houseId.substring(1, houseId.length() - 1);
            checkArray = ids.split(",");
            String codes = houseCode.substring(1, houseCode.length() - 1);
            checkCodeArray = codes.split(",");
        }

        String archiveIds = "";
        // 在把新的仓房
        if (checkArray != null && checkArray.length > 0) {
            Date date = new Date();
            StorageArchive storageArchive = new StorageArchive();
            for (int i = 0; i < checkArray.length; i++) {
                storageArchive = new StorageArchive();
                storageArchive.setId(UUID.randomUUID().toString());
                storageArchive.setOrgId(orgId);
                storageArchive.setStorehouseId(checkArray[i]);
                storageArchive.setStorehouseCode(checkCodeArray[i].substring(1, checkCodeArray[i].length() - 1));
                storageArchive.setCreaterId(userId);
                storageArchive.setCreater(userName);
                storageArchive.setCreateTime(date);
                storageArchive.setArchiveState(archiveState);
                StorageArchiveMapper.insertSelective(storageArchive);
                archiveIds += storageArchive.getId() + ",";
            }
        }
        return archiveIds;
    }

    @Override
    public List<StorageArchive> getArchiveViewList(String storehouseId, String startTime, String endTime, String orgId) {
        StorageArchiveExample example = new StorageArchiveExample();
        StorageArchiveExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(orgId);
        if (ParameterUtil.isnotnull(storehouseId)) {
            criteria.andStorehouseIdEqualTo(storehouseId);
        }
        if (ParameterUtil.isnotnull(startTime) && ParameterUtil.isnotnull(endTime)) {
            criteria.andCreateTimeBetween(ParameterUtil.string2datetime(startTime + " 00:00:00"),ParameterUtil.string2datetime(endTime + " 23:59:59"));
        }
        criteria.andArchiveStateEqualTo("5");
        example.setOrderByClause("create_time desc");
        return StorageArchiveMapper.selectByExample(example);
    }

    @Override
    public Map<Integer,Object> getArchiveDataList(String orgId) {
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("orgId", orgId);
        List<Map<String, Object>> archiveHouse = StorageArchiveMapper.getArchiveDataList(map);
        archiveHouse = ParameterUtil.slashCapitals(archiveHouse);

        Map<Integer, Object> objMap = new HashMap<Integer, Object>();
        for (Map<String, Object> obj : archiveHouse) {
            objMap.put(Integer.parseInt(obj.get("storehouseId").toString()), obj);
        }
        return objMap;
    }

    @Override
    public List<Map<String,Object>> getCRKArchiveList(String orgId) {
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("orgId", orgId);
        return ParameterUtil.slashCapitals(StorageArchiveMapper.getCRKArchiveList(map));
    }
}
