package com.chinaitop.depot.agent.basic.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import com.chinaitop.depot.agent.basic.model.*;
import com.chinaitop.depot.agent.basic.service.agentDepotService;
import com.chinaitop.depot.agent.basic.service.agentStoreWareService;
import com.chinaitop.depot.agent.basic.service.agentStorehouseService;
import com.chinaitop.depot.utils.ImportServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.agent.basic.service.agentService;
import com.chinaitop.utils.ParameterUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.web.multipart.MultipartFile;

@RestController
@RequestMapping(value = "/agent")
@Api(description = "代储点信息管理")
public class AgentController {

    @Resource
    private agentService agentService;


    @Resource
    private agentDepotService agentDepotService;

    @Resource
    private agentStorehouseService agentStorehouseService;

    @Resource
    private agentStoreWareService agentStoreWareService;

    @Resource
    private ImportServiceImpl importService;

    /**
     * 查询代储点信息记录
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getAgentList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储点信息", notes = "代储点信息记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "agentName", value = "代储点名称", paramType = "query"),
            @ApiImplicitParam(name = "orgid", value = "组织id", paramType = "query")
    })
    public PageInfo<BusinessAgent> getAgentList(Integer pageNum, Integer pageSize,String agentName,String orgid){

        BusinessAgentExample example = new BusinessAgentExample();
        BusinessAgentExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(Integer.valueOf(orgid));
        criteria.andDelFlagEqualTo(1);
        if(ParameterUtil.isnotnull(agentName)){
            criteria.andAgentNameLike("%"+agentName+"%");
        }
        example.setOrderByClause("updatetime desc");
        PageHelper.startPage(pageNum, pageSize);
        List<BusinessAgent> list = agentService.getAgentList(example);
        PageInfo<BusinessAgent> pageInfo = new PageInfo<BusinessAgent>(list);
        return pageInfo;
    }

    /**
     * 代储点查询信息--代储库列表转化用
     * @return
     */
    @RequestMapping(value = "/getQueryAgentList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储点查询信息", notes = "代储点信息记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public Map<Integer, Object> getQueryAgentList(String orgId){
        BusinessAgentExample example = new BusinessAgentExample();
        BusinessAgentExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(Integer.valueOf(orgId));
        criteria.andDelFlagEqualTo(1);
        List<BusinessAgent> list = agentService.getAgentList(example);
        Map<Integer, Object> agentData = new HashMap<Integer, Object>();
        for (BusinessAgent obj : list) {
            agentData.put(obj.getId(), obj);
        }
        return agentData;
    }

    /**
     * 保存代储点信息
     * @param agentJson 主键id
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/saveAgent", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="保存代储点信息", notes = "保存代储点详细信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "agentJson", value = "代储点信息", paramType = "form")
    })
    public Map<String, Object> saveAgent(String agentJson,Integer orgId) throws Exception {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        BusinessAgent agent = (BusinessAgent)mapper.readValue(agentJson, BusinessAgent.class);
        if (agent.getId() == null) {
            agent.setOrgId(orgId);
            agent.setDelFlag(1);
            agent.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//创建时间
            agentService.add(agent);
        } else {
            agentService.update(agent);
        }
        modelMap.put("agentId", agent.getId());
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 删除代储点信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/removeAgent", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="删除代储点信息", notes = "删除代储点信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "form")
    })
    public Map<String, Object> removeAgent(Integer id){
        Map<String, Object> modelMap = new HashMap<>();
        if (id != null) {
//            this.removeDepotList(id);
            agentService.updateDeleteById(id);
            modelMap.put("status", "success");
        }
        return modelMap;
    }

    /**
     * 根据id查找代储点信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="根据id查找代储点信息", notes = "根据id查找代储点信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
    })
    public BusinessAgent edit(Integer id) {
        BusinessAgent agent = new BusinessAgent();
        if (id != null) {
            agent = agentService.findById(id);
        }
        return agent;
    }


    //根据库id删除粮库
    public void removeDepotList(Integer agentId){
        BusinessAgentDepotExample example = new BusinessAgentDepotExample();
        BusinessAgentDepotExample.Criteria criteria = example.createCriteria();
        criteria.andAgentIdEqualTo(agentId.toString());
        List<BusinessAgentDepot> depotList = agentDepotService.getAgentDepotList(example);
        for (BusinessAgentDepot depotData:depotList) {
            agentDepotService.updateDeleteById(depotData.getId(),2);

            this.removeWareList(depotData.getId());
        }
    }
    //根据库id删除仓房
    public void removeStoreList(Integer depotId){
        BusinessAgentStorehouseExample example = new BusinessAgentStorehouseExample();
        BusinessAgentStorehouseExample.Criteria criteria = example.createCriteria();
        criteria.andAgentDepotIdEqualTo(depotId);
        List<BusinessAgentStorehouse> storeList = agentStorehouseService.getStoreDataList(example);
        for (BusinessAgentStorehouse storeData:storeList) {
            agentStorehouseService.updateStoreDeleteById(storeData.getId(),2);

            this.removeWareList(storeData.getId());
        }
    }

    //根据仓房id删除货位
    public void removeWareList(Integer storeHouseId){
        BusinessAgentWarehouseExample example = new BusinessAgentWarehouseExample();
        BusinessAgentWarehouseExample.Criteria criteria = example.createCriteria();
        criteria.andAgentStorehouseIdEqualTo(storeHouseId);
        List<BusinessAgentWarehouse> wareList = agentStoreWareService.getWareDataList(example);
        for (BusinessAgentWarehouse wareData:wareList) {
            agentStoreWareService.removeAgentWare(wareData.getId(),2);
        }
    }

    /**
     * 导入excel
     * @param file 文件
     * @param fileType 类型：如xlsx
     * @param orgId 组织机构id
     * @return
     */
    @RequestMapping(value="/importFile", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="导入excel", notes = "导入excel")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file", value = "文件数据", paramType = "query"),
            @ApiImplicitParam(name = "fileType", value = "文件名", paramType = "query"),
            @ApiImplicitParam(name = "rootPath", value = "路径", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织机构id", paramType = "query")
    })
    public Map<String, Object> importFile(MultipartFile file, String fileType, HttpServletRequest request, Integer orgId){

        String[] titleArr = {"代储点名称", "法定代表人", "统一社会信用代码", "代储点性质", "代储点类型", "仓容",
                "人员", "信用", "经度", "纬度", "所属行政区划分"};//第一行的title
        boolean[] emptyArr = {false, false, false, false, false, false,
                true, true, false, true, true};//指定列可为空，不可为空值为false.

        List<List<String>> list = new ArrayList<List<String>>();//该list为数据返回的list
        Map<String, Object> fileMap = new HashMap<>();
        String errormsg = null;
        try {
        	errormsg = importService.importFile(file, fileType, request, titleArr, emptyArr, list,0);

            if (errormsg != null && !"".equals(errormsg)) {
                fileMap.put("errormsg",errormsg);
                return fileMap;
            }

            if (list.size()>0) {
                errormsg = agentService.importData(list, orgId);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        fileMap.put("errormsg",errormsg);
        return fileMap;
    }
}
