package com.chinaitop.depot.agent.basic.controller;

import com.alibaba.fastjson.JSONArray;
import com.chinaitop.depot.agent.basic.model.*;
import com.chinaitop.depot.agent.basic.service.agentDepotService;
import com.chinaitop.depot.agent.basic.service.agentStoreWareService;
import com.chinaitop.depot.agent.basic.service.agentStorehouseService;
import com.chinaitop.depot.agent.basic.service.agentTankExistService;
import com.chinaitop.depot.utils.ImportServiceImpl;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;

@RestController
@RequestMapping(value = "/agentDepot")
@Api(description = "代储点信息管理")
public class AgentDepotController {

    @Resource
    private agentDepotService agentDepotService;

    @Resource
    private agentStorehouseService agentStorehouseService;

    @Resource
    private agentTankExistService agentTankExistService;

    @Resource
    private agentStoreWareService agentStoreWareService;

    @Resource
    private ImportServiceImpl importService;

    /**
     * 代储库信息信息记录
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getAgentDepotList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储库信息", notes = "代储点粮库信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "agentDepotName", value = "代储库名称", paramType = "query"),
            @ApiImplicitParam(name = "agentId", value = "代储点id", paramType = "query"),
            @ApiImplicitParam(name = "orgid", value = "组织id", paramType = "query")
    })
    public PageInfo<BusinessAgentDepot> getAgentDepotList(Integer pageNum, Integer pageSize,String agentDepotName,String orgid, String agentId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgid);
        if(ParameterUtil.isnotnull(agentDepotName)){
            map.put("agentDepotName","%" + agentDepotName + "%");
        }
        if(ParameterUtil.isnotnull(agentId)){
            map.put("agentId",agentId);
        }

        PageHelper.startPage(pageNum, pageSize);
        List<BusinessAgentDepot> list = agentDepotService.getAgentList(map);
        PageInfo<BusinessAgentDepot> pageInfo = new PageInfo<BusinessAgentDepot>(list);
        return pageInfo;
    }

    /**
     * 代储库信息信息记录--仓房用
     * @return
     */
    @RequestMapping(value = "/getAgentDepotHouse", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储库信息", notes = "代储点粮库信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "agentDepotName", value = "代储库名称", paramType = "query"),
            @ApiImplicitParam(name = "agentId", value = "代储点id", paramType = "query"),
            @ApiImplicitParam(name = "orgid", value = "组织id", paramType = "query")
    })
    public List<BusinessAgentDepot> getAgentDepotHouse(String agentDepotName,String orgid, String agentId){
        BusinessAgentDepotExample example = new BusinessAgentDepotExample();
        BusinessAgentDepotExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(Integer.parseInt(orgid));
        criteria.andDepotIdEqualTo(0);
        criteria.andDelFlagEqualTo(1);
        if(ParameterUtil.isnotnull(agentDepotName)){
            criteria.andAgentDepotNameLike("%" + agentDepotName + "%");
        }
        if(ParameterUtil.isnotnull(agentId)){
            criteria.andAgentIdEqualTo(agentId);
        }

        List<BusinessAgentDepot> list = agentDepotService.getAgentDepotList(example);
        return list;
    }

    /**
     * 保存代储点信息
     * @param agentJson 主键id
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/saveAgentDepot", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="保存代储点信息", notes = "保存代储点详细信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "agentJson", value = "代储库信息", paramType = "form")
    })
    public Map<String, Object> saveAgentDepot(String agentJson,Integer orgId) throws Exception {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        BusinessAgentDepot agentDepot = (BusinessAgentDepot)mapper.readValue(agentJson, BusinessAgentDepot.class);
        agentDepot.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//修改时间
        if (agentDepot.getId() == null) {
            agentDepot.setOrgId(orgId);
            agentDepot.setDelFlag(1);
            agentDepot.setDepotId(0);
            agentDepotService.add(agentDepot);
        } else {
            agentDepotService.update(agentDepot);
        }
        modelMap.put("agentId", agentDepot.getId());
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 删除代储库信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/removeAgentDepot", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="删除代储库信息", notes = "删除代储库信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "form")
    })
    public Map<String, Object> removeAgentDepot(Integer id){
        Map<String, Object> modelMap = new HashMap<>();
        if (id != null) {
            agentDepotService.updateDeleteById(id,2);
            modelMap.put("status", "success");
        }
        return modelMap;
    }

    /**
     * 根据id查找代储点信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/editDepot", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="根据id查找代储点信息", notes = "根据id查找代储点信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
    })
    public BusinessAgentDepot editDepot(Integer id) {
        BusinessAgentDepot agentDepot = new BusinessAgentDepot();
        if (id != null) {
            agentDepot = agentDepotService.findById(id);
        }
        return agentDepot;
    }

    /**
     * 代储库查询信息--列表转化用
     * @return
     */
    @RequestMapping(value = "/getQueryAgentDepotList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储库查询信息", notes = "代储库信息记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public Map<Integer, Object> getQueryAgentDepotList(String orgId,Integer depotId){
        BusinessAgentDepotExample example = new BusinessAgentDepotExample();
        BusinessAgentDepotExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(Integer.valueOf(orgId));
        criteria.andDelFlagEqualTo(1);
        if(depotId!=null && depotId!=000){
        	criteria.andDepotIdEqualTo(depotId);
        }
        if(depotId!=null && depotId==000){ //去掉手动新增的库点
        	criteria.andDepotIdNotEqualTo(0);
        }
        List<BusinessAgentDepot> list = agentDepotService.getAgentDepotList(example);
        Map<Integer, Object> agentData = new HashMap<Integer, Object>();
        for (BusinessAgentDepot obj : list) {
            agentData.put(obj.getId(), obj);
        }
        return agentData;
    }

    /**
     * 查找仓房所有粮库
     * @param orgName 组织机构名称
     * @return
     */
    @RequestMapping(value="/getHouseDepotList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="查找仓房所有粮库", notes = "查找仓房所有粮库信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name="orgId", value="组织机构id", paramType="query"),
            @ApiImplicitParam(name="storehouseId", value="仓房id", paramType="query"),
            @ApiImplicitParam(name="orgName", value="组织机构名称", paramType="query")
    })
    public List<List<Map<String, Object>>> getHouseDepotList(String orgId,String storehouseId,String orgName) {
        //查询所有粮库
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        map.put("storehouseId",storehouseId);

        if(ParameterUtil.isnotnull(orgName)){
            map.put("orgName","%" + orgName + "%");
        }
        //查出来的数据
        List<Map<String, Object>> list = agentDepotService.getHouseDepotList(map);
        //准备处理，每六条数据一个list
        List<List<Map<String, Object>>> dataList = new ArrayList<List<Map<String, Object>>>();
        List<Map<String, Object>> six = new ArrayList<Map<String, Object>>();
        for (int i = 0;i<list.size();i++){
            six.add(list.get(i));
            //如果查出来的数据小于六条则直接加入总数据返回
            if(ParameterUtil.isequal(i+1,list.size())){
                dataList.add(six);
                break;
            }
            if(ParameterUtil.isequal((i+1)%6,0)){
                dataList.add(six);
                six = new ArrayList<Map<String, Object>>();;
            }
        }
        return dataList;
    }

    /**
     * 查找油罐所有粮库
     * @param orgName 组织机构名称
     * @return
     */
    @RequestMapping(value="/getTankDepotList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="查找油罐所有粮库", notes = "查找油罐所有粮库信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name="orgId", value="组织机构id", paramType="query"),
            @ApiImplicitParam(name="TankId", value="油罐id", paramType="query"),
            @ApiImplicitParam(name="orgName", value="组织机构名称", paramType="query")
    })
    public List<List<Map<String, Object>>> getTankDepotList(String orgId,String TankId,String orgName) {
        //查询所有粮库
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        map.put("TankId",TankId);

        if(ParameterUtil.isnotnull(orgName)){
            map.put("orgName","%" + orgName + "%");
        }
        //查出来的数据
        List<Map<String, Object>> list = agentDepotService.getTankDepotList(map);
        //准备处理，每六条数据一个list
        List<List<Map<String, Object>>> dataList = new ArrayList<List<Map<String, Object>>>();
        List<Map<String, Object>> six = new ArrayList<Map<String, Object>>();
        for (int i = 0;i<list.size();i++){
            six.add(list.get(i));
            //如果查出来的数据小于六条则直接加入总数据返回
            if(ParameterUtil.isequal(six.size(),list.size())){
                dataList.add(six);
            }
            if(ParameterUtil.isequal((i+1)%6,0)){
                dataList.add(six);
                six = new ArrayList<Map<String, Object>>();;
            }
        }
        return dataList;
    }

    /**
     * 保存已有粮库代储库信息
     * @param agentDepotId 选中粮库
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/saveLaoHouseDepot", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="保存已有粮库代储库信息-仓房", notes = "保存已有粮库代储库信息-仓房")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "agentDepotId", value = "已有粮库", paramType = "form"),
            @ApiImplicitParam(name = "storehouseId", value = "仓房id", paramType = "form"),
            @ApiImplicitParam(name = "warehouIds", value = "货位id集合", paramType = "form"),
            @ApiImplicitParam(name = "orgName", value = "组织结构名称", paramType = "form"),
            @ApiImplicitParam(name = "orgId", value = "组织结构id", paramType = "form")
    })
    public Map<String, Object> saveLaoHouseDepot(Integer agentDepotId,Integer storehouseId,String warehouIds,String orgName,Integer orgId) throws Exception {
        //添加代储库
        Map<String, Object> modelMap = new HashMap<String, Object>();
        //查询是否已经添加过此粮库，如果添加过则过滤，不再添加
        BusinessAgentDepotExample example = new BusinessAgentDepotExample();
        BusinessAgentDepotExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(agentDepotId);
        criteria.andDepotIdEqualTo(orgId);
        List<BusinessAgentDepot> depotNum = agentDepotService.selectIdNum(example);
        //添加仓房
        BusinessAgentStorehouse agentStorehouse = new BusinessAgentStorehouse();
        agentStorehouse.setOrgId(agentDepotId);
        agentStorehouse.setDelFlag(1);

        if(depotNum.size() == 0){
            BusinessAgentDepot agentDepot = new BusinessAgentDepot();
            agentDepot.setOrgId(agentDepotId);
            agentDepot.setDelFlag(1);
            agentDepot.setAgentDepotName(orgName);
            agentDepot.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//修改时间
            agentDepot.setDepotId(orgId);
            agentDepotService.add(agentDepot);
            agentStorehouse.setAgentDepotId(agentDepot.getId());
        }else if(depotNum.get(0).getDelFlag() == 2){
            agentDepotService.updateDeleteById(depotNum.get(0).getId(),1);
            agentStorehouse.setAgentDepotId(depotNum.get(0).getId());
        }else{
            agentStorehouse.setAgentDepotId(depotNum.get(0).getId());
        }

        agentStorehouse.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//修改时间
        agentStorehouse.setStorehouseId(storehouseId);
        agentStorehouseService.add(agentStorehouse);

        //添加货位
        String[] wareIds = warehouIds.split("\\|");
        for (String wareId: wareIds ) {
            BusinessAgentWarehouse agentWarehouse = new BusinessAgentWarehouse();
            agentWarehouse.setWarehouseId(Integer.parseInt(wareId));
            agentWarehouse.setAgentStorehouseId(agentStorehouse.getId());
            agentWarehouse.setOrgId(agentDepotId);
            agentWarehouse.setDelFlag(1);
            agentWarehouse.setCreateTime(ParameterUtil.string2datetime(ParameterUtil.getSysDateTime()));//修改时间
            agentWarehouse.setBgmxzTbsj(ParameterUtil.string2datetime(ParameterUtil.getSysDateTime()));
            agentWarehouse.setQccrkTbsj(ParameterUtil.string2datetime(ParameterUtil.getSysDateTime()));
            agentWarehouse.setHccrkTbsj(ParameterUtil.string2datetime(ParameterUtil.getSysDateTime()));
            agentStoreWareService.addWareHouse(agentWarehouse);
        }
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 保存已有粮库代储库信息-油罐
     * @param agentDepotId 选中粮库
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/saveLaoTankDepot", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="保存已有粮库代储库信息-油罐", notes = "保存已有粮库代储库信息-油罐")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "agentDepotId", value = "已有粮库", paramType = "form"),
            @ApiImplicitParam(name = "tankId", value = "油罐id", paramType = "form"),
            @ApiImplicitParam(name = "orgName", value = "组织结构名称", paramType = "form"),
            @ApiImplicitParam(name = "orgId", value = "组织结构id", paramType = "form")
    })
    public Map<String, Object> saveLaoTankDepot(Integer agentDepotId,String tankId,String orgName,Integer orgId) throws Exception {
        //添加代储库
        Map<String, Object> modelMap = new HashMap<String, Object>();
        //查询是否已经添加过此粮库，如果添加过则过滤，不再添加
        BusinessAgentDepotExample example = new BusinessAgentDepotExample();
        BusinessAgentDepotExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(agentDepotId);
        criteria.andDepotIdEqualTo(orgId);
        List<BusinessAgentDepot> depotNum = agentDepotService.selectIdNum(example);
        //添加仓房
        BusinessAgentTankExist agentTankExist = new BusinessAgentTankExist();
        agentTankExist.setOrgId(agentDepotId);
        agentTankExist.setDelFlag(1);
        agentTankExist.setBgmxzTbsj(new Date());
        agentTankExist.setHccrkTbsj(new Date());
        agentTankExist.setQccrkTbsj(new Date());


        if(depotNum.size() == 0){
            BusinessAgentDepot agentDepot = new BusinessAgentDepot();
            agentDepot.setOrgId(agentDepotId);
            agentDepot.setDelFlag(1);
            agentDepot.setAgentDepotName(orgName);
            agentDepot.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//修改时间
            agentDepot.setDepotId(orgId);
            agentDepotService.add(agentDepot);
            agentTankExist.setAgentDepotId(agentDepot.getId());
        }else if(depotNum.get(0).getDelFlag() == 2){
            agentDepotService.updateDeleteById(depotNum.get(0).getId(),1);
            agentTankExist.setAgentDepotId(depotNum.get(0).getId());
        }else{
            agentTankExist.setAgentDepotId(depotNum.get(0).getId());
        }

        agentTankExist.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//修改时间
        agentTankExist.setTankId(tankId);
        agentTankExist.setId(ParameterUtil.getCode());
        agentTankExistService.add(agentTankExist);
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 保存粮库代储企业信息
     * @param agentId 选中粮库
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/addAgentId", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="保存粮库代储企业信息", notes = "保存粮库代储企业信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "代储库id", paramType = "form"),
            @ApiImplicitParam(name = "agentId", value = "代储企业id", paramType = "form")
    })
    public Map<String, Object> addAgentId(Integer id,String agentId) throws Exception {
        //添加代储库
        Map<String, Object> modelMap = new HashMap<String, Object>();
        BusinessAgentDepot agentDepot = new BusinessAgentDepot();
        agentDepot.setId(id);
        agentDepot.setAgentId(agentId);
        agentDepotService.updateByPrimaryKeySelective(agentDepot);
        modelMap.put("status", "success");
        return modelMap;
    }

    //根据油罐id删除油罐
    @RequestMapping(value="/removeTank", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="删除油罐", notes = "根据油罐id删除油罐")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "depotId", value = "代储粮库id", paramType = "form"),
            @ApiImplicitParam(name = "tankId", value = "油罐id", paramType = "form")
    })
    public void removeTank(Integer depotId,String tankId){
        Integer houseNum = agentDepotService.getIdHouse(depotId);
        if(houseNum == 1){
            agentDepotService.updateDeleteById(depotId,2);
        }
        agentTankExistService.updateTankDeleteById(tankId,2);
    }

    /**
     * 导入excel
     * @param file 文件
     * @param fileType 类型：如xlsx
     * @param orgId 组织机构id
     * @return
     */
    @RequestMapping(value="/importDepotFile", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="导入excel", notes = "导入excel")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file", value = "文件数据", paramType = "query"),
            @ApiImplicitParam(name = "fileType", value = "文件名", paramType = "query"),
            @ApiImplicitParam(name = "rootPath", value = "路径", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织机构id", paramType = "query")
    })
    public Map<String, Object> importDepotFile(MultipartFile file, String fileType, HttpServletRequest request, Integer orgId){

        String[] titleArr = {"库点名称","代储点名称", "库点类别", "设计仓容", "地址","电话","经度", "纬度", "所属行政区划分"};//第一行的title
        boolean[] emptyArr = {false, false, false, true, true, true, true, true, false};//指定列可为空，不可为空值为false.

        List<List<String>> list = new ArrayList<List<String>>();//该list为数据返回的list
        Map<String, Object> fileMap = new HashMap<>();
        String errormsg = null;
        try {
        	errormsg = importService.importFile(file, fileType, request, titleArr, emptyArr, list,0);

            if (errormsg != null && !"".equals(errormsg)) {
                fileMap.put("errormsg",errormsg);
                return fileMap;
            }

            if (list.size()>0) {
                errormsg = agentDepotService.importData(list, orgId);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        fileMap.put("errormsg",errormsg);
        return fileMap;
    }

    //根据库id删除仓房
    public void removeStoreList(Integer depotId){
        BusinessAgentStorehouseExample example = new BusinessAgentStorehouseExample();
        BusinessAgentStorehouseExample.Criteria criteria = example.createCriteria();
        criteria.andAgentDepotIdEqualTo(depotId);
        List<BusinessAgentStorehouse> storeList = agentStorehouseService.getStoreDataList(example);
        for (BusinessAgentStorehouse storeData:storeList) {
            agentStorehouseService.updateStoreDeleteById(storeData.getId(),2);

            this.removeWareList(storeData.getId());
        }
    }

    //根据仓房id删除货位
    public void removeWareList(Integer storeHouseId){
        BusinessAgentWarehouseExample example = new BusinessAgentWarehouseExample();
        BusinessAgentWarehouseExample.Criteria criteria = example.createCriteria();
        criteria.andAgentStorehouseIdEqualTo(storeHouseId);
        List<BusinessAgentWarehouse> wareList = agentStoreWareService.getWareDataList(example);
        for (BusinessAgentWarehouse wareData:wareList) {
            agentStoreWareService.removeAgentWare(wareData.getId(),2);
        }
    }
}
