package com.chinaitop.depot.agent.basic.controller;

import com.chinaitop.depot.agent.basic.model.BusinessAgentPeople;
import com.chinaitop.depot.agent.basic.model.BusinessAgentPeopleExample;
import com.chinaitop.depot.agent.basic.service.agentPeopleService;
import com.chinaitop.depot.utils.ImportServiceImpl;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/agentPeople")
@Api(description = "人员信息管理")
public class AgentPeopleController {

    @Resource
    private agentPeopleService agentPeopleService;

    /**
     * 查询人员信息记录
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getAgentPeopleList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="人员信息", notes = "人员信息记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "PeopleName", value = "人员名称", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public PageInfo<BusinessAgentPeople> getAgentPeopleList(Integer pageNum, Integer pageSize,String PeopleName,String orgId){

        BusinessAgentPeopleExample example = new BusinessAgentPeopleExample();
        BusinessAgentPeopleExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(Integer.valueOf(orgId));
        if(ParameterUtil.isnotnull(PeopleName)){
            criteria.andPeopleNameLike("%"+PeopleName+"%");
        }
        example.setOrderByClause("updatetime desc");
        PageHelper.startPage(pageNum, pageSize);
        List<BusinessAgentPeople> list = agentPeopleService.getAgentList(example);
        PageInfo<BusinessAgentPeople> pageInfo = new PageInfo<BusinessAgentPeople>(list);
        return pageInfo;
    }

    /**
     * 保存人员信息
     * @param agentJson 主键id
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/saveAgentPeople", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="保存人员信息", notes = "保存人员详细信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "agentJson", value = "人员信息", paramType = "form"),
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public Map<String, Object> saveAgentPeople(String agentJson,Integer orgId) throws Exception {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        BusinessAgentPeople agentPeople = (BusinessAgentPeople)mapper.readValue(agentJson, BusinessAgentPeople.class);
        agentPeople.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//修改时间
        if (agentPeople.getId() == null) {
            agentPeople.setOrgId(orgId);
            agentPeopleService.add(agentPeople);
        } else {
            agentPeopleService.update(agentPeople);
        }
        modelMap.put("agentId", agentPeople.getId());
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 删除代储点信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/removeAgentPeople", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="删除人员信息", notes = "删除人员信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "form")
    })
    public Map<String, Object> removeAgentPeople(Integer id){
        Map<String, Object> modelMap = new HashMap<>();
        if (id != null) {
            agentPeopleService.updateDeleteById(id);
            modelMap.put("status", "success");
        }
        return modelMap;
    }

    /**
     * 根据id查找人员信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="根据id查找人员信息", notes = "根据id查找人员信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
    })
    public BusinessAgentPeople edit(Integer id) {
        BusinessAgentPeople agentPeople = new BusinessAgentPeople();
        if (id != null) {
            agentPeople = agentPeopleService.findById(id);
        }
        return agentPeople;
    }
}
