package com.chinaitop.depot.agent.basic.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.agent.basic.model.BusinessAgentWarehouse;
import com.chinaitop.depot.agent.basic.service.agentStoreWareService;
import com.chinaitop.utils.ParameterUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value = "/agentStoreWare")
@Api(description = "代储货位信息管理")
public class AgentStoreWareController {

    @Resource
    private agentStoreWareService agentStoreWareService;

    /**
     * 货位列表
     * @return
     */
    @RequestMapping(value = "/getWareList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="货位列表", notes = "货位信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "warehouseName", value = "货位名称", paramType = "query"),
            @ApiImplicitParam(name = "storehouseId", value = "仓房id", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public PageInfo<Map<String,Object>> getWareList(Integer pageNum, Integer pageSize, String warehouseName, String storehouseId, String orgId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        if(ParameterUtil.isnotnull(warehouseName)){
            map.put("warehouseName","%" + warehouseName + "%");
        }
        if(ParameterUtil.isnotnull(storehouseId)){
            map.put("storehouseId",storehouseId);
        }

        PageHelper.startPage(pageNum, pageSize);
        List<Map<String,Object>> list = agentStoreWareService.getWareList(map);
        PageInfo<Map<String,Object>> pageInfo = new PageInfo<Map<String,Object>>(list);
        //处理驼峰命名大写问题
        List<Map<String, Object>> page_list = ParameterUtil.slashCapitals(pageInfo.getList());
        pageInfo.setList(page_list);
        return pageInfo;
    }

    /**
     * 根据agent_storehouse_id(仓房id)和orgId 获取货位的列表
     */
    @RequestMapping(value = "/getAgentWarehouseList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储库的货位信息", notes = "代储库的货位信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query"),
            @ApiImplicitParam(name = "agentStorehouseId", value = "仓房id", paramType = "query")
    })
    public List<Map<String, Object>> getAgentWarehouseList(Integer orgId,Integer agentStorehouseId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        map.put("agentStorehouseId",agentStorehouseId);
        List<Map<String, Object>> list = agentStoreWareService.getAgentStoreWare(map);
        //处理驼峰命名大写问题
        List<Map<String, Object>> listDatas = ParameterUtil.slashCapitals(list);
        return listDatas;
    }

    /**
     * 查询已有货位
     */
    @RequestMapping(value="/getWarehouse", method = RequestMethod.GET)
    @ApiOperation(value="根据单位和仓房查询货位信息（货位对外接口）", notes = "根据单位和仓房查询货位信息（货位对外接口）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "unitId", value = "单位ID", paramType = "query"),
            @ApiImplicitParam(name = "storehouseId", value = "仓房ID", paramType = "query")
    })
    public Map<String, Object> getWarehouse(Integer unitId, Integer storehouseId) {
        Map<String, Object> map = null;

        map = agentStoreWareService.getWarehouse(unitId, storehouseId);
        //处理驼峰命名大写问题
        Map<String, Object> mapData = ParameterUtil.slashCapitals(map);
        return mapData;
    }
    
    /**
     * 除去代储货位的本库货位
     */
    @RequestMapping(value = "/getAgentWareHouseId", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="除去代储货位的本库货位", notes = "除去代储货位的本库货位")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public List<Integer> getAgentwareHouseId(Integer orgId){
        return agentStoreWareService.getAgentwareHouseId(orgId);
    }
}
