package com.chinaitop.depot.agent.basic.controller;

import com.alibaba.fastjson.JSONArray;
import com.chinaitop.depot.agent.basic.model.BusinessAgentStorehouse;
import com.chinaitop.depot.agent.basic.model.BusinessAgentWarehouse;
import com.chinaitop.depot.agent.basic.model.BusinessAgentWarehouseExample;
import com.chinaitop.depot.agent.basic.service.agentDepotService;
import com.chinaitop.depot.agent.basic.service.agentStoreWareService;
import com.chinaitop.depot.agent.basic.service.agentStorehouseService;
import com.chinaitop.utils.ParameterUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/agentStorehouse")
@Api(description = "代储仓房信息管理")
public class AgentStorehouseController {

    @Resource
    private agentDepotService agentDepotService;

    @Resource
    private agentStorehouseService agentStorehouseService;

    @Resource
    private agentStoreWareService agentStoreWareService;

    /**
     * 代储库信息
     * @return
     */
    @RequestMapping(value = "/getDepotInfo", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储库信息", notes = "代储点粮库信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public List<Map<String,Object>> getDepotInfo(String orgId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        List<Map<String,Object>> list = agentDepotService.getDepotLaoList(map);
        //处理驼峰命名大写问题
        List<Map<String, Object>> listDatas = ParameterUtil.slashCapitals(list);
        return listDatas;
    }

    /**
     * 代储仓房信息
     * @return
     */
    @RequestMapping(value = "/getAgentStoreInfo", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储仓房信息", notes = "代储点仓房信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query"),
            @ApiImplicitParam(name = "depotId", value = "代储库id", paramType = "query")
    })
    public List<Map<String,Object>> getAgentStoreInfo(String depotId, String orgId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        map.put("depotId",depotId);
        List<Map<String,Object>> list = agentStorehouseService.getAgentStoreInfo(map);
        //处理驼峰命名大写问题
        List<Map<String, Object>> listDatas = ParameterUtil.slashCapitals(list);
        return listDatas;
    }

    /**
     * 获取代储库下边的仓房信息，已有粮库和手加粮库都有
     * @return
     */
    @RequestMapping(value = "/getDepotToStoreList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储仓房信息", notes = "代储点仓房信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query"),
            @ApiImplicitParam(name = "depotId", value = "代储库id", paramType = "query")
    })
    public List<Map<String,Object>> getDepotToStoreList(String orgId, String depotId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        map.put("depotId",depotId);
        List<Map<String,Object>> list = agentStorehouseService.getDepotToStoreList(map);
        //处理驼峰命名大写问题
        List<Map<String, Object>> listDatas = ParameterUtil.slashCapitals(list);
        return listDatas;
    }

    /**
     * 仓房列表
     * @return
     */
    @RequestMapping(value = "/getStoreList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="仓房列表", notes = "仓房信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "agentDepotName", value = "代储库名称", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public PageInfo<Map<String,Object>> getStoreList(Integer pageNum, Integer pageSize, String agentDepotName, String orgId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        if(ParameterUtil.isnotnull(agentDepotName)){
            map.put("agentDepotName","%" + agentDepotName + "%");
        }

        PageHelper.startPage(pageNum, pageSize);
        List<Map<String,Object>> list = agentStorehouseService.getStoreList(map);
        PageInfo<Map<String,Object>> pageInfo = new PageInfo<Map<String,Object>>(list);
        //处理驼峰命名大写问题
        List<Map<String, Object>> page_list = ParameterUtil.slashCapitals(pageInfo.getList());
        pageInfo.setList(page_list);
        return pageInfo;
    }

    /**
     * 查找粮库下仓房
     * @param orgId 组织机构名称
     * @return
     */
    @RequestMapping(value="/getStorehouseList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="查找粮库下仓房", notes = "查找粮库下仓房信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name="orgId", value="组织机构id", paramType="query"),
            @ApiImplicitParam(name="depotId", value="代储点Id", paramType="query")
    })
    public List<List<Map<String, Object>>> getStorehouseList(String orgId,String depotId) {
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        if(ParameterUtil.isnotnull(depotId)){
            map.put("depotId",depotId);
        }
        //查出来的数据
        List<Map<String, Object>> list = agentStorehouseService.getStorehouseList(map);
        //处理驼峰命名大写问题
        List<Map<String, Object>> listDatas = ParameterUtil.slashCapitals(list);
        //准备处理，每六条数据一个list
        List<List<Map<String, Object>>> dataList = new ArrayList<List<Map<String, Object>>>();
        List<Map<String, Object>> six = new ArrayList<Map<String, Object>>();
        for (int i = 0;i<listDatas.size();i++){
            six.add(listDatas.get(i));
            //如果查出来的数据小于六条则直接加入总数据返回
            if(ParameterUtil.isequal(six.size(),listDatas.size())){
                dataList.add(six);
            }
            if(ParameterUtil.isequal((i+1)%6,0)){
                dataList.add(six);
                six = new ArrayList<Map<String, Object>>();;
            }
        }
        return dataList;
    }

    /**
     * 根据agent_depot_id、orgId 获取仓房信息
     * @return
     */
    @RequestMapping(value = "/getAgentStorehouseInfo", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储仓房信息", notes = "代储点仓房信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public List<Map<String, Object>> getAgentStorehouseInfo(Integer orgId,Integer agentDepotId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        map.put("agentDepotId",agentDepotId);
        List<Map<String, Object>> list = agentStorehouseService.getAgentStorehouse(map);
        //处理驼峰命名大写问题
        List<Map<String, Object>> listDatas = ParameterUtil.slashCapitals(list);
        return listDatas;
    }

    //根据仓房id删除仓房
    @RequestMapping(value="/removeStoreHouse", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="删除仓房", notes = "根据仓房id删除仓房")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "depotId", value = "代储粮库id", paramType = "form"),
            @ApiImplicitParam(name = "storehouseId", value = "仓房id", paramType = "form")
    })
    public void removeStoreHouse(Integer depotId,Integer storehouseId){
        agentDepotService.updateDeleteById(depotId,2);
        agentStorehouseService.updateStoreDeleteById(storehouseId,2);
        this.removeWareList(storehouseId);
    }

    //根据仓房id删除货位
    public void removeWareList(Integer storeHouseId){
        BusinessAgentWarehouseExample example = new BusinessAgentWarehouseExample();
        BusinessAgentWarehouseExample.Criteria criteria = example.createCriteria();
        criteria.andAgentStorehouseIdEqualTo(storeHouseId);
        List<BusinessAgentWarehouse> wareList = agentStoreWareService.getWareDataList(example);
        for (BusinessAgentWarehouse wareData:wareList) {
            agentStoreWareService.removeAgentWare(wareData.getId(),2);
        }
    }
    
    
    /**
     * 获取本库的代储仓房(启用着的)
     * @return
     */
    @RequestMapping(value = "/getAgentHouseId", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="获取本库的代储仓房(启用着的)", notes = "获取本库的代储仓房(启用着的)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public List<Integer> getAgentHouseId(Integer orgId){
         
        return agentDepotService.getAgentHouseId(orgId);
    }
    
}
