package com.chinaitop.depot.agent.basic.controller;

import com.chinaitop.depot.agent.basic.model.BusinessAgentTank;
import com.chinaitop.depot.agent.basic.model.BusinessAgentTankExample;
import com.chinaitop.depot.agent.basic.service.agentTankService;
import com.chinaitop.depot.agent.basic.service.agentTankService;
import com.chinaitop.depot.utils.ImportServiceImpl;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/agentTank")
@Api(description = "油罐信息管理")
public class AgentTankController {

    @Resource
    private agentTankService agentTankService;

    @Resource
    private ImportServiceImpl importService;

    /**
     * 查询油罐信息记录
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getAgentTankList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="油罐信息", notes = "油罐信息记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "tankNumber", value = "油罐编号", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public PageInfo<BusinessAgentTank> getAgentList(Integer pageNum, Integer pageSize,String tankNumber,String orgId){

        BusinessAgentTankExample example = new BusinessAgentTankExample();
        BusinessAgentTankExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(Integer.valueOf(orgId));
        if(ParameterUtil.isnotnull(tankNumber)){
            criteria.andTankNumberLike("%" + tankNumber +"%");
        }
        example.setOrderByClause("updatetime desc");
        PageHelper.startPage(pageNum, pageSize);
        List<BusinessAgentTank> list = agentTankService.getAgentList(example);
        PageInfo<BusinessAgentTank> pageInfo = new PageInfo<BusinessAgentTank>(list);
        return pageInfo;
    }

    /**
     * 保存代储点信息
     * @param agentJson 主键id
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/saveAgentTank", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="保存油罐信息", notes = "保存油罐详细信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "agentJson", value = "油罐信息", paramType = "form"),
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public Map<String, Object> saveAgent(String agentJson,Integer orgId) throws Exception {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        BusinessAgentTank agent = (BusinessAgentTank)mapper.readValue(agentJson, BusinessAgentTank.class);
        agent.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//修改时间
        if (ParameterUtil.isnull(agent.getId())) {
            agent.setOrgId(orgId);
            agentTankService.add(agent);
        } else {
            agentTankService.update(agent);
        }
        modelMap.put("agentId", agent.getId());
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 删除油罐信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/removeAgentTank", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="删除油罐信息", notes = "删除油罐信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "form")
    })
    public Map<String, Object> removeAgent(Integer id){
        Map<String, Object> modelMap = new HashMap<>();
        if (id != null) {
            agentTankService.updateDeleteById(id);
            modelMap.put("status", "success");
        }
        return modelMap;
    }

    /**
     * 根据id查找代储点信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="根据id查找油罐信息", notes = "根据id查找油罐信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
    })
    public BusinessAgentTank edit(Integer id) {
        BusinessAgentTank agent = new BusinessAgentTank();
        if (id != null) {
            agent = agentTankService.findById(id);
        }
        return agent;
    }

    /**
     * 导入excel
     * @param file 文件
     * @param fileType 类型：如xlsx
     * @param orgId 组织机构id
     * @return
     */
    @RequestMapping(value="/importFile", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="导入excel", notes = "导入excel")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file", value = "文件数据", paramType = "query"),
            @ApiImplicitParam(name = "fileType", value = "文件名", paramType = "query"),
            @ApiImplicitParam(name = "rootPath", value = "路径", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织机构id", paramType = "query")
    })
    public Map<String, Object> importFile(MultipartFile file, String fileType, HttpServletRequest request, Integer orgId){

        String[] titleArr = {"粮库名称","油罐编号", "设计容量", "设计年限", "是否国债投资", "启用日期", "当前状态", "责任保管员"};//第一行的title
        boolean[] emptyArr = {false, false, false, false, false, false, false, false};//指定列可为空，不可为空值为false.

        List<List<String>> list = new ArrayList<List<String>>();//该list为数据返回的list
        Map<String, Object> fileMap = new HashMap<>();
        String errormsg = null;
        try {
        	errormsg = importService.importFile(file, fileType, request, titleArr, emptyArr, list,0);

            if (errormsg != null && !"".equals(errormsg)) {
                fileMap.put("errormsg",errormsg);
                return fileMap;
            }

            if (list.size()>0) {
                errormsg = agentTankService.importData(list, orgId);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        fileMap.put("errormsg",errormsg);
        return fileMap;
    }
    
    
    /**
     * 综合业务查询代储的油罐信息
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getTankList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="综合业务查询代储的油罐信息", notes = "综合业务查询代储的油罐信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "depotId", value = "代储库的库id", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public List<Map<String, Object>> getAgentTankInfo(Integer depotId,Integer orgId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        map.put("depotId",depotId);
        List<Map<String, Object>> list = agentTankService.getAgentTank(map);
        return list;
    }
    
    
    /**
     * 根据agent_depot_id、orgId 获取油罐信息
     * @return
     */
    @RequestMapping(value = "/getTankExistInfo", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储油罐信息", notes = "代储点油罐信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "组织id", paramType = "query")
    })
    public List<Map<String, Object>> getTankExistInfo(Integer orgId,Integer agentDepotId){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgId);
        map.put("agentDepotId",agentDepotId);
        List<Map<String, Object>> list = agentTankService.getTankExistInfo(map);
        return list;
    }
}
