package com.chinaitop.depot.agent.basic.service.impl;

import com.chinaitop.depot.agent.basic.mapper.BusinessAgentDepotMapper;
import com.chinaitop.depot.agent.basic.mapper.BusinessAgentTankMapper;
import com.chinaitop.depot.agent.basic.model.BusinessAgentDepot;
import com.chinaitop.depot.agent.basic.model.BusinessAgentDepotExample;
import com.chinaitop.depot.agent.basic.model.BusinessAgentTank;
import com.chinaitop.depot.agent.basic.model.BusinessAgentTankExample;
import com.chinaitop.depot.agent.basic.service.agentTankService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class AgentTankServiceImpl implements agentTankService {

    @Resource
    private BusinessAgentTankMapper BusinessAgentTankMapper;

    @Resource
    private BusinessAgentDepotMapper BusinessAgentDepotMapper;

    /**
     * 查询仓储处作业记录
     * * @param map
     * @return
     */
    @Override
    public List<BusinessAgentTank> getAgentList(BusinessAgentTankExample map) {
        return BusinessAgentTankMapper.selectByExample(map);
    }

    @Override
    public void add(BusinessAgentTank agent) {
        BusinessAgentTankMapper.insert(agent);
    }

    @Override
    public void update(BusinessAgentTank camera) {
        BusinessAgentTankMapper.updateByPrimaryKey(camera);
    }

    @Override
    public void updateDeleteById(Integer agentId) {
        BusinessAgentTankMapper.deleteByPrimaryKey(agentId);
    }

    @Override
    public BusinessAgentTank findById(Integer id) {
        return BusinessAgentTankMapper.selectByPrimaryKey(id);
    }

    @Override
    public String importData(List<List<String>> list, Integer orgId) {
        Date date = new Date();
        BusinessAgentTank agent = null;

        String msg = "";
        int len = 0;

        for (int i = 0; i < list.size(); i++) {
            List<String> revList = list.get(i);

            for (int j = 0; j < revList.size(); j++) {
                switch(j) {
                    case 0:
                        //粮库名称
                        agent = new BusinessAgentTank();
                        if (revList.get(j) != null && !"".equals(revList.get(j))) {

                            //根据粮库名称查询出来相关的粮库id
                            BusinessAgentDepotExample example = new BusinessAgentDepotExample();
                            BusinessAgentDepotExample.Criteria criteria = example.createCriteria();
                            criteria.andOrgIdEqualTo(orgId);
                            criteria.andDelFlagEqualTo(1);//未删除
                            criteria.andAgentDepotNameEqualTo(revList.get(j));
                            List<BusinessAgentDepot> agentList = BusinessAgentDepotMapper.selectByExample(example);
                            if(agentList.size() == 0){
                                msg = "第 "+(i+1)+" 行第 1 列的值的粮库相关信息没有找到，请确定是否有！";
                                return msg;
                            }

                            agent.setDepotId(agentList.get(0).getId());
                        }
                        break;
                    case 1:
                        //油罐编号
                        if (revList.get(j) != null && !"".equals(revList.get(j))) {
                            len = revList.get(j).length();
                            if (len > 30) {
                                msg = "第 "+(i+1)+" 行第 2 列的值的长度不能超过30！";
                                return msg;
                            }
                            agent.setTankNumber(revList.get(j));
                        }
                        agent.setCreateTime(date);//修改时间
                        agent.setOrgId(orgId);
                        break;
                    case 2:
                        //设计容量
                        if (revList.get(j) != null && !"".equals(revList.get(j))) {
                            len = revList.get(j).length();
                            if (len > 30) {
                                msg = "第 "+(i+1)+" 行第 3 列的值的长度不能超过30！";
                                return msg;
                            }
                        }
                        agent.setDesignStorage(revList.get(j));
                        break;
                    case 3:
                        //设计年限
                        if (revList.get(j) != null && !"".equals(revList.get(j))) {
                            len = revList.get(j).length();
                            if (len > 30) {
                                msg = "第 "+(i+1)+" 行第 4 列的值的长度不能超过30！";
                                return msg;
                            }
                        }
                        agent.setDesignYear(revList.get(j));
                        break;
                    case 4:
                        //是否国债投资
                        String sf = revList.get(j);
                        if ("是".equals(sf) || "是" == sf) {
                            agent.setTreasuryBonds("true");
                        }else{
                            agent.setTreasuryBonds("false");
                        }
                        break;
                    case 5:
                        //启用日期
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+1)+" 行第 6 列的值的长度不能超过20！";
                            return msg;
                        }
                        agent.setEnableDate(revList.get(j));
                        break;
                    case 6:
                        //当前状态
                        switch(revList.get(j)) {
                            case "完好":
                                agent.setState("3011");
                                break;
                            case "需小修":
                                agent.setState("3012");
                                break;
                            case "需大修":
                                agent.setState("3013");
                                break;
                            case "待报废":
                                agent.setState("3014");
                                break;
                            case "待拆除":
                                agent.setState("3015");
                                break;
                            case "死角仓":
                                agent.setState("3016");
                                break;
                            case "其他":
                                agent.setState("3017");
                                break;
                            default:
                                msg = "第 "+(i+1)+" 行第 7 列的值并没有在项目中存在请查询后再导入！";
                                return msg;
                        }
                        break;
                    case 7:
                        //责任保管员
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+1)+" 行第 8 列的值的长度不能超过20！";
                            return msg;
                        }
                        agent.setKeeper(revList.get(j));
                        break;
                    default :
                        break;
                }

            }
            BusinessAgentTankMapper.insert(agent);
        }
        return msg;
    }

	@Override
	public List<Map<String, Object>> getAgentTank(Map<String, Object> map) {
		// TODO Auto-generated method stub
		return BusinessAgentTankMapper.getAgentTank(map);
	}
	
	@Override
	public List<Map<String, Object>> getTankExistInfo(Map<String, Object> map) {
		// TODO Auto-generated method stub
		return BusinessAgentTankMapper.getTankExistInfo(map);
	}
}
