package com.chinaitop.depot.agent.bgz.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.agent.bgz.model.BusinessAgentFcbgz;
import com.chinaitop.depot.agent.bgz.model.BusinessAgentFcbgzExample;
import com.chinaitop.depot.agent.bgz.model.BusinessAgentFcbgzExample.Criteria;
import com.chinaitop.depot.agent.bgz.service.BusinessAgentBgmxzService;
import com.chinaitop.depot.agent.bgz.service.BusinessAgentFcbgzService;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value = "/agent_fcbgz")
@Api(description = "代储库分仓保管账控制类")
public class BusinessAgentFcbgzController {

	@Resource
	private BusinessAgentFcbgzService fcbgzService;

	@Resource
	private BusinessAgentBgmxzService bgmxzService;
	
	@RequestMapping(value = "/getList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="代储库分仓保管账信息列表", notes = "代储库分仓保管账信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "fcbgzJson", value = "条件对象", paramType = "query")
    })
	public PageInfo<BusinessAgentFcbgz> list(Integer pageNum, Integer pageSize, String fcbgzJson) {
		
		ObjectMapper mapper = new ObjectMapper();
		BusinessAgentFcbgz fcbgz = null;
		try {
			fcbgz = (BusinessAgentFcbgz) mapper.readValue(fcbgzJson, BusinessAgentFcbgz.class);
		} catch (Exception e) {
			e.printStackTrace();
		}
		BusinessAgentFcbgzExample example = new BusinessAgentFcbgzExample();
		Criteria criteria = example.createCriteria();

		/* orgId */
		if(ParameterUtil.isnotnull(fcbgz.getOrgId())){
			criteria.andOrgIdEqualTo(fcbgz.getOrgId());
		}
		/* 代储点ID */
		if(ParameterUtil.isnotnull(fcbgz.getAgentId())){
			criteria.andAgentIdEqualTo(fcbgz.getAgentId());
		}
		/* 代储库ID */
		if(ParameterUtil.isnotnull(fcbgz.getDckId())){
			criteria.andDckIdEqualTo(fcbgz.getDckId());
		}
		/* 仓房ID */
		if(ParameterUtil.isnotnull(fcbgz.getHouseId())){
			criteria.andHouseIdEqualTo(fcbgz.getHouseId());
		}
		/* 货位ID */
		if(ParameterUtil.isnotnull(fcbgz.getWareId())){
			criteria.andWareIdEqualTo(fcbgz.getWareId());
		}
		/* 粮食品种ID */
		if(ParameterUtil.isnotnull(fcbgz.getLspz())){
			criteria.andLspzEqualTo(fcbgz.getLspz());
		}
		/* 粮食性质ID */
		if(ParameterUtil.isnotnull(fcbgz.getLsxz())){
			criteria.andLsxzEqualTo(fcbgz.getLsxz());
		}
		/* 业务时间 */
		if(ParameterUtil.isnotnull(fcbgz.getFssj())){
			criteria.andFssjEqualTo(fcbgz.getFssj());
		}

		/* 分页 */
		if(ParameterUtil.isnotnull(pageNum) && ParameterUtil.isnotnull(pageSize)){
			PageHelper.startPage(pageNum, pageSize);
		}
		List<BusinessAgentFcbgz> list = fcbgzService.findByConditions(example);
		PageInfo<BusinessAgentFcbgz> pageInfo = new PageInfo<BusinessAgentFcbgz>(list);
		
		return pageInfo;
	}

	@RequestMapping(value = "/edit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="新增或修改代储库分仓保管账信息", notes = "新增或修改代储库分仓保管账信息")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "fcbgzJson", value = "数据对象", paramType = "form"),
        @ApiImplicitParam(name = "type", value = "操作类型", paramType = "form")
    })
	public Map<String, Object> edit(String fcbgzJson, String type) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		try {
			BusinessAgentFcbgz fcbgz = (BusinessAgentFcbgz) mapper.readValue(fcbgzJson, BusinessAgentFcbgz.class);
			if (type != null && "shenhe".equals(type)) {
				fcbgzService.update(fcbgz);
			} else {
				/**
				 * 实现思路
				 * 1、先查询有没有记过账
				 * 2、记过账那么修改数据就行，没记过账就新增
				 * 3、记账后要把这个业务时间段的保管明细数据设置为已记账
				 */
				BusinessAgentFcbgzExample example = new BusinessAgentFcbgzExample();
				Criteria criteria = example.createCriteria();
				/* orgId */
				if(ParameterUtil.isnotnull(fcbgz.getOrgId())){
					criteria.andOrgIdEqualTo(fcbgz.getOrgId());
				}
				/* 代储点ID */
				if(ParameterUtil.isnotnull(fcbgz.getAgentId())){
					criteria.andAgentIdEqualTo(fcbgz.getAgentId());
				}
				/* 代储库ID */
				if(ParameterUtil.isnotnull(fcbgz.getDckId())){
					criteria.andDckIdEqualTo(fcbgz.getDckId());
				}
				/* 仓房ID */
				if(ParameterUtil.isnotnull(fcbgz.getHouseId())){
					criteria.andHouseIdEqualTo(fcbgz.getHouseId());
				}
				/* 货位ID */
				if(ParameterUtil.isnotnull(fcbgz.getWareId())){
					criteria.andWareIdEqualTo(fcbgz.getWareId());
				}
				/* 粮食品种ID */
				if(ParameterUtil.isnotnull(fcbgz.getLspz())){
					criteria.andLspzEqualTo(fcbgz.getLspz());
				}
				/* 粮食性质ID */
				if(ParameterUtil.isnotnull(fcbgz.getLsxz())){
					criteria.andLsxzEqualTo(fcbgz.getLsxz());
				}
				/* 业务时间 */
				if(ParameterUtil.isnotnull(fcbgz.getFssj())){
					criteria.andFssjEqualTo(fcbgz.getFssj());
				}
				List<BusinessAgentFcbgz> list = fcbgzService.findByConditions(example);

				/* 如果记过账了那么就修改，否则新增 */
				if (null != list && list.size() != 0) {
					fcbgzService.update(fcbgz);
				} else {
					//新增
					fcbgzService.insert(fcbgz);
				}

				//记账后将这天的保管明细账数据设置为已记账，记账后不可修改
				fcbgzService.updateSfjz(fcbgz);
			}
			modelMap.put("status", "success");
			modelMap.put("msg", "操作成功！");
		} catch (Exception e) {
			e.printStackTrace();
			modelMap.put("status", "error");
			modelMap.put("msg", "操作失败！");
		}
		return modelMap;
	}
	
	@RequestMapping(value = "/remove", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="删除选中分仓保管账信息", notes = "删除选中分仓保管账信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "数据ID", paramType = "form")
    })
	public Map<String, Object> remove(Integer id) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		try {
			if (null != id) {
				fcbgzService.remove(id);
				modelMap.put("status", "success");
				modelMap.put("msg", "操作成功！");
			}
		} catch (Exception e) {
			e.printStackTrace();
			modelMap.put("status", "error");
			modelMap.put("msg", "操作失败！");
		}
		return modelMap;
	}
	
	@RequestMapping(value="/getBgzzList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="保管总账列表，按月统计", notes = "保管总账列表，按月统计，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name="bgzzJson", value="数据对象", paramType="query")
	})
	public PageInfo<Map<String, Object>> getBgzzList(Integer pageNum, Integer pageSize, String fcbgzJson) {

		Map<String, Object> paramMap = new HashMap<String, Object>();
		ObjectMapper mapper = new ObjectMapper();
		BusinessAgentFcbgz fcbgz = null;
		try {
			fcbgz = (BusinessAgentFcbgz) mapper.readValue(fcbgzJson, BusinessAgentFcbgz.class);
		} catch (Exception e) {
			e.printStackTrace();
		}
		if(ParameterUtil.isnotnull(fcbgz.getOrgId())){
			paramMap.put("orgId", fcbgz.getOrgId());
		}
		if(ParameterUtil.isnotnull(fcbgz.getAgentId())){
			paramMap.put("agentId", fcbgz.getAgentId());
		}
		if(ParameterUtil.isnotnull(fcbgz.getDckId())){
			paramMap.put("dckId", fcbgz.getDckId());
		}
		if(ParameterUtil.isnotnull(fcbgz.getLspz())){
			paramMap.put("lspz", fcbgz.getLspz());
		}
		if(ParameterUtil.isnotnull(fcbgz.getLsxz())){
			paramMap.put("lsxz", fcbgz.getLsxz());
		}
		if(ParameterUtil.isnotnull(fcbgz.getFssj())){
			paramMap.put("fssj", fcbgz.getFssj());
		}
		/* 分页 */
		if(ParameterUtil.isnotnull(pageNum) && ParameterUtil.isnotnull(pageSize)){
			PageHelper.startPage(pageNum, pageSize);
		}

		List<Map<String, Object>> list = fcbgzService.selectBgzzGroupByList(paramMap);

		PageInfo<Map<String, Object>> pageInfo = new PageInfo<Map<String, Object>>(list);
		list = pageInfo.getList();
		if (null != list && list.size() > 0) {
			Map<String, Object> param_map = null;
			for (Map<String, Object> map : list) {
				map.put("fssj", map.get("fssj").toString().substring(0, 7));
				param_map = fcbgzService.selectBgzzList(map);
				if (null != param_map) {
					map.put("agent_name", param_map.get("agent_name"));
					map.put("agent_depot_name", param_map.get("agent_depot_name"));
					map.put("kcsl", param_map.get("kcsl"));
				}
			}
			list = ParameterUtil.slashCapitals(list);
			pageInfo.setList(list);
			
		}
		
		return pageInfo;
	}
}
