package com.chinaitop.depot.agent.bgz.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.chinaitop.depot.agent.feign.AgileHouseFeignClient;
import org.springframework.stereotype.Service;

import com.chinaitop.depot.agent.basic.mapper.BusinessAgentDepotMapper;
import com.chinaitop.depot.agent.basic.mapper.BusinessAgentWarehouseMapper;
import com.chinaitop.depot.agent.basic.model.BusinessAgentDepot;
import com.chinaitop.depot.agent.basic.model.BusinessAgentDepotExample;
import com.chinaitop.depot.agent.basic.model.BusinessAgentWarehouse;
import com.chinaitop.depot.agent.basic.model.BusinessAgentWarehouseExample;
import com.chinaitop.depot.agent.bgz.mapper.BusinessAgentBgmxzMapper;
import com.chinaitop.depot.agent.bgz.model.BusinessAgentBgmxz;
import com.chinaitop.depot.agent.bgz.service.BusinessAgentBgmxzService;

@Service
public class BusinessAgentBgmxzServiceImpl implements BusinessAgentBgmxzService {

	@Resource
	private BusinessAgentBgmxzMapper bgmxzMapper;
	
	@Resource
	private BusinessAgentDepotMapper agentDepotMapper;
	
	@Resource
	private BusinessAgentWarehouseMapper agentWareHouseMapper;

	@Resource
	private AgileHouseFeignClient agileHouseFeignClient;

	@Override
	public List<Map<String, Object>> findByConditions(Map<String, Object> parem_map) {
		//获取查询条件去出入库查数据
		List<Map<String, Object>> wareTimeList = bgmxzMapper.selectAgentHouseWareList(parem_map);
		String str = JSON.toJSONString(wareTimeList);
		//调用depot-agile接口
		JSONArray crkobj =  agileHouseFeignClient.queryFcbgzData(str);
		//转换出入库查到的数据
		List<Map<String, Object>> listObjectFir = (List<Map<String, Object>>) JSONArray.parse(String.valueOf(crkobj));
		//查询代储点手动添加的数据
		List<Map<String, Object>> agentHc = bgmxzMapper.selectByExampleNew(parem_map);
		//两个list集合相加
		agentHc.addAll(listObjectFir);
		//返回集合
		return agentHc;
	}

	@Override
	public BusinessAgentBgmxz findByParmaryKey(Integer id) {
		BusinessAgentBgmxz bgmxz = bgmxzMapper.selectByPrimaryKey(id);
		return bgmxz;
	}

	@Override
	public void insert(BusinessAgentBgmxz bgmxz) {
		bgmxzMapper.insertSelective(bgmxz);
	}

	@Override
	public void update(BusinessAgentBgmxz bgmxz) {
		bgmxzMapper.updateByPrimaryKey(bgmxz);
	}

	@Override
	public void remove(Integer id) {
		bgmxzMapper.deleteByPrimaryKey(id);
	}

	@Override
	public Map<String, Object> selectDetail(Map<String, Object> map) {
		
		Map<String, Object> list = bgmxzMapper.selectDetail(map);
		return list;
	}

	@Override
	public List<Map<String, Object>> selectAgentHouseWareList(Map<String, Object> paramMap) {
		List<Map<String, Object>> list = bgmxzMapper.selectAgentHouseWareList(paramMap);
		return list;
	}

	@Override
	public BusinessAgentDepot selectAgentDepotObj(Integer orgId, Integer dckId) {
		BusinessAgentDepotExample example = new BusinessAgentDepotExample();
		BusinessAgentDepotExample.Criteria criteria = example.createCriteria();
		criteria.andOrgIdEqualTo(orgId);
		criteria.andDepotIdEqualTo(dckId);
		criteria.andDelFlagEqualTo(1);
		List<BusinessAgentDepot> list = agentDepotMapper.selectByExample(example);
		BusinessAgentDepot agentDepot = new BusinessAgentDepot();
		if (null != list && list.size() > 0) {
			agentDepot = list.get(0);
		}
		return agentDepot;
	}

	@Override
	public void updateAgentDepot(Integer orgId) {
		//要修改的值
		BusinessAgentWarehouse record = new BusinessAgentWarehouse();
		record.setBgmxzTbsj(new Date());
		//修改条件
		BusinessAgentWarehouseExample example = new BusinessAgentWarehouseExample();
		BusinessAgentWarehouseExample.Criteria criteria = example.createCriteria();
		criteria.andOrgIdEqualTo(orgId);
		criteria.andDelFlagEqualTo(1);
		agentWareHouseMapper.updateByExampleSelective(record, example);
	}

	@Override
	public List<Map<String, Object>> selectFcbgzGroupBy(Map<String, Object> map) {
		List<Map<String, Object>> list = bgmxzMapper.selectFcbgzGroupBy(map);
		return list;
	}

}
