package com.chinaitop.depot.agent.crk.service.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.chinaitop.depot.agent.feign.AgileHouseFeignClient;
import org.springframework.stereotype.Service;

import com.chinaitop.depot.agent.basic.mapper.BusinessAgentMapper;
import com.chinaitop.depot.agent.basic.mapper.BusinessAgentWarehouseMapper;
import com.chinaitop.depot.agent.basic.model.BusinessAgentWarehouse;
import com.chinaitop.depot.agent.basic.model.BusinessAgentWarehouseExample;
import com.chinaitop.depot.agent.crk.mapper.BusinessAgentQcMapper;
import com.chinaitop.depot.agent.crk.model.BusinessAgentQc;
import com.chinaitop.depot.agent.crk.service.agentQcService;
import com.chinaitop.utils.ParameterUtil;

@Service
public class AgentQcServiceImpl implements agentQcService {

    @Resource
    private BusinessAgentQcMapper businessAgentQcMapper;

    @Resource
    private BusinessAgentMapper BusinessAgentMapper;

    @Resource
	private BusinessAgentWarehouseMapper agentWareHouseMapper;

    @Resource
    private AgileHouseFeignClient agileHouseFeignClient;

    /**
     * 查询仓储处作业记录
     * * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getAgentQcList(Map<String, Object> map) {
        //获取查询条件去出入库查数据
        List<Map<String, Object>> wareTimeList = businessAgentQcMapper.selectAgentHouseWareList(map);
        String str = JSON.toJSONString(wareTimeList);
        //调用depot-agile接口
        JSONArray crkobj =  agileHouseFeignClient.queryQcCrkRecordData(str);
        //转换出入库查到的数据
        List<Map<String, Object>> listObjectFir = (List<Map<String, Object>>) JSONArray.parse(String.valueOf(crkobj));
        //查询代储点手动添加的数据
        List<Map<String, Object>> agentQc = businessAgentQcMapper.getDataByExample(map);
        //两个list集合相加
        agentQc.addAll(listObjectFir);
        //返回集合
        return agentQc;
    }

    @Override
    public void add(BusinessAgentQc agent) {
        businessAgentQcMapper.insert(agent);
    }

    @Override
    public void update(BusinessAgentQc camera) {
        businessAgentQcMapper.updateByPrimaryKey(camera);
    }

    @Override
    public void updateDeleteById(Integer agentId) {
        businessAgentQcMapper.deleteByPrimaryKey(agentId);
    }

    @Override
    public BusinessAgentQc findById(Integer id) {
        return businessAgentQcMapper.selectByPrimaryKey(id);
    }

    @Override
    public String importData(List<List<String>> list, Integer orgId, Map<String, Object> agentMap) {
        Date date = new Date();
        BusinessAgentQc agent = null;
        Map<String,Object> enumData = new HashMap<>();

        String msg = "";
        int len = 0;

        for (int i = 0; i < list.size(); i++) {
            List<String> revList = list.get(i);

            for (int j = 0; j < revList.size(); j++) {
                switch(j) {
                    case 0:
                        //通知单编号
                        agent = new BusinessAgentQc();
                        if (ParameterUtil.isnotnull(revList.get(j))) {
                            len = revList.get(j).length();
                            if (len > 50) {
                                msg = "第 "+(i+3)+" 行第 1 列的值的长度不能超过50！";
                                return msg;
                            }
                            agent.setNotice(revList.get(j));
                        }
                        agent.setCreateTime(date);//创建时间
                        agent.setOrgId(orgId);
                        break;
                    case 1:
                        //入站单号
                        if (ParameterUtil.isnotnull(revList.get(j))) {
                            len = revList.get(j).length();
                            if (len > 50) {
                                msg = "第 "+(i+3)+" 行第 2 列的值的长度不能超过50！";
                                return msg;
                            }
                        }
                        agent.setBusinessNumber(revList.get(j));
                        break;
                    case 2:
                        //火车皮号
                        if (ParameterUtil.isnotnull(revList.get(j))) {
                            len = revList.get(j).length();
                            if (len > 50) {
                                msg = "第 "+(i+3)+" 行第 3 列的值的长度不能超过50！";
                                return msg;
                            }
                        }
                        agent.setCarBrand(revList.get(j));
                        break;
                    case 3:
                        //业务类型
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 4 列的值的长度不能超过20！";
                            return msg;
                        }
                        agent.setBusinessType(revList.get(j));
                        break;
                    case 4:
                        //粮油品种
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 5 列的值的长度不能超过20！";
                            return msg;
                        }
                        enumData.put("enumName", revList.get(j));
                        enumData.put("parentId", "1061");
                        Map<String,Object> enumId = BusinessAgentMapper.getEnumId(enumData);
                        if(enumId.size() == 0){
                            msg = "第 "+(i+3)+" 行第 5 列的值的不正确，请查询后再添加！";
                            return msg;
                        }
                        agent.setLspz(Integer.parseInt(enumId.get("enumid").toString()));
                        break;
                    case 5:
                        //数量（公斤）
                        len = revList.get(j).length();
                        if (len > 30) {
                            msg = "第 "+(i+3)+" 行第 6 列的值的长度不能超过30！";
                            return msg;
                        }
                        agent.setGrainNumber(revList.get(j));
                        break;
                    case 6:
                        //仓房名称
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 7 列的值的长度不能超过20！";
                            return msg;
                        }
                        enumData.put("storehouseName", revList.get(j));
                        enumData.put("depotId", agentMap.get("depotId"));
                        Map<String,Object> storehouseId = BusinessAgentMapper.getStorehouseId(enumData);
                        if(ParameterUtil.isnull(storehouseId)){
                            msg = "第 "+(i+3)+" 行第 7 列的值的不正确，请查询后再添加！";
                            return msg;
                        }
                        agent.setStorehouseId(Integer.parseInt(storehouseId.get("storehouseid").toString()));
                        break;
                    case 7:
                        //货位名称
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 8 列的值的长度不能超过20！";
                            return msg;
                        }
                        enumData.put("warehouseName", revList.get(j));
                        Map<String,Object> warehouseId = BusinessAgentMapper.getWarehouseId(enumData);
                        if(ParameterUtil.isnull(warehouseId)){
                            msg = "第 "+(i+3)+" 行第 8 列的值的不正确，请查询后再添加！";
                            return msg;
                        }
                        agent.setWarehouseId(Integer.parseInt(warehouseId.get("warehouseid").toString()));
                        break;
                    case 8:
                        //出入库日期
                        len = revList.get(j).length();
                        if (len > 100) {
                            msg = "第 "+(i+3)+" 行第 5 列的值的长度不能超过100！";
                            return msg;
                        }
                        agent.setCrkTime(ParameterUtil.string2date(revList.get(j)));
                        break;
                    default :
                        break;
                }
            }
            agent.setAgentId(Integer.parseInt(agentMap.get("agentid").toString()));
            agent.setAgentDepotId(Integer.parseInt(agentMap.get("depotid").toString()));
            businessAgentQcMapper.insert(agent);
        }
        return msg;
    }

	@Override
	public void updateAgentDepot(Integer orgId) {
		//要修改的值
		BusinessAgentWarehouse record = new BusinessAgentWarehouse();
		record.setQccrkTbsj(new Date());
		//修改条件
		BusinessAgentWarehouseExample example = new BusinessAgentWarehouseExample();
		BusinessAgentWarehouseExample.Criteria criteria = example.createCriteria();
		criteria.andOrgIdEqualTo(orgId);
		criteria.andDelFlagEqualTo(1);
		agentWareHouseMapper.updateByExampleSelective(record, example);
	}

	@Override
	public List<Map<String, Object>> selectAgentHouseWareList(Map<String, Object> paramMap) {
		List<Map<String, Object>> list = businessAgentQcMapper.selectAgentHouseWareList(paramMap);
		return list;
	}
}
