package com.chinaitop.depot.agent.number.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.chinaitop.depot.agent.basic.model.BusinessAgentDepot;
import com.chinaitop.depot.agent.basic.service.agentService;
import com.chinaitop.depot.agent.bgz.service.BusinessAgentBgmxzService;
import com.chinaitop.depot.agent.number.model.BusinessAgentNumber;
import com.chinaitop.depot.agent.number.model.BusinessAgentNumberExample;
import com.chinaitop.depot.agent.number.model.BusinessAgentNumberExample.Criteria;
import com.chinaitop.depot.agent.number.service.agentNumberService;
import com.chinaitop.depot.utils.ImportServiceImpl;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

@RestController
@RequestMapping(value = "/agentNumber")
@Api(description = "库存数量管理")
public class AgentNumberController {

    @Resource
    private agentNumberService agentNumberService;

    @Resource
    private agentService agentService;

    @Resource
    private ImportServiceImpl importService;

    @Resource
    private BusinessAgentBgmxzService bgmxzService;

    /**
     * 查询库存数量信息记录
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getAgentNumberList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="库存数量信息", notes = "查询库存数量信息记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "agentId", value = "代储点id", paramType = "query"),
            @ApiImplicitParam(name = "agentDepotId", value = "代储库id", paramType = "query"),
            @ApiImplicitParam(name = "storehouseId", value = "仓房id", paramType = "query"),
            @ApiImplicitParam(name = "warehouseId", value = "货位id", paramType = "query"),
            @ApiImplicitParam(name = "orgid", value = "组织id", paramType = "query")
    })
    public PageInfo<Map<String, Object>> getAgentNumberList(Integer pageNum, Integer pageSize, Integer agentId, Integer agentDepotId,
                                                                        Integer storehouseId, Integer warehouseId, String orgid){
        Map<String,Object> map = new HashMap<>();
        map.put("orgId",orgid);
        if(ParameterUtil.isnotnull(agentId)){
            map.put("agentId",agentId);
        }
        if(ParameterUtil.isnotnull(agentDepotId)){
            map.put("agentDepotId",agentDepotId);
        }
        if(ParameterUtil.isnotnull(storehouseId)){
            map.put("storehouseId",storehouseId);
        }
        if(ParameterUtil.isnotnull(warehouseId)){
            map.put("warehouseId",warehouseId);
        }

        PageHelper.startPage(pageNum, pageSize);
        List<Map<String, Object>> list = agentNumberService.getAgentNumberList(map);
        PageInfo<Map<String, Object>> pageInfo = new PageInfo<Map<String, Object>>(list);
        //处理驼峰命名大写问题
        List<Map<String, Object>> page_list = ParameterUtil.slashCapitals(pageInfo.getList());
        pageInfo.setList(page_list);
        //计算返回页数
        pageInfo.setPageNum(pageNum);
        pageInfo.setPageSize(pageSize);
        //由于结果是>startRow的，所以实际的需要+1
        if (pageInfo.getSize() == 0) {
            pageInfo.setStartRow(0);
            pageInfo.setEndRow(0);
        } else {
            pageInfo.setStartRow(pageInfo.getStartRow() + 1);
            //计算实际的endRow（最后一页的时候特殊）
            pageInfo.setEndRow(pageInfo.getStartRow() - 1 +pageInfo.getSize());
        }
        return pageInfo;
    }

    /**
     * 保存库存数量信息
     * @param agentNumberJson 主键id
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/saveAgent", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="保存库存数量", notes = "保存库存数量信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "agentNumberJson", value = "库存数量信息", paramType = "form")
    })
    public Map<String, Object> saveAgent(String agentNumberJson,Integer orgId) throws Exception {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        BusinessAgentNumber agent = (BusinessAgentNumber)mapper.readValue(agentNumberJson, BusinessAgentNumber.class);
        if (ParameterUtil.isnull(agent.getId())) {
            agent.setOrgId(orgId);
            agent.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//创建时间
            agentNumberService.add(agent);
        } else {
            agentNumberService.update(agent);
        }
        modelMap.put("agentId", agent.getId());
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 删除库存数量信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/removeAgentNumber", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="删除库存数量信息", notes = "删除库存数量信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "库存数量主键id", paramType = "form")
    })
    public Map<String, Object> removeAgentNumber(Integer id){
        Map<String, Object> modelMap = new HashMap<>();
        if (id != null) {
            agentNumberService.updateDeleteById(id);
            modelMap.put("status", "success");
        }
        return modelMap;
    }

    /**
     * 根据id查找库存数量信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/getAgentNumberEdit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="根据id查找库存数量信息", notes = "根据id查找库存数量信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "库存数量主键id", paramType = "query")
    })
    public BusinessAgentNumber getAgentNumberEdit(Integer id) {
        BusinessAgentNumber agent = new BusinessAgentNumber();
        if (id != null) {
            agent = agentNumberService.findById(id);
        }
        return agent;
    }

    /**
     * 导入excel
     * @param file 文件
     * @param fileType 类型：如xlsx
     * @param orgId 组织机构id
     * @return
     */
    @RequestMapping(value="/importFile", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value="导入excel", notes = "导入excel")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file", value = "文件数据", paramType = "query"),
            @ApiImplicitParam(name = "fileType", value = "文件名", paramType = "query"),
            @ApiImplicitParam(name = "rootPath", value = "路径", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织机构id", paramType = "query")
    })
    public Map<String, Object> importFile(MultipartFile file, String fileType, HttpServletRequest request, Integer orgId){

        String[] titleArr = {"仓房名称", "货位名称", "粮油品种", "粮油性质", "等级","收获年度", "库存数量（公斤）"};//第一行的title
        boolean[] emptyArr = {false, false, false, false, false,false, false};//指定列可为空，不可为空值为false.

        List<List<String>> list = new ArrayList<List<String>>();//该list为数据返回的list
        Map<String, Object> fileMap = new HashMap<>();
        String errormsg = null;
        try {
        	errormsg = importService.importFile(file, fileType, request, titleArr, emptyArr, list,1);

            if (errormsg != null && !"".equals(errormsg)) {
                fileMap.put("errormsg",errormsg);
                return fileMap;
            }

            //查询代储点名称和代储库名称对应的id
            Map<String, Object> agentMap = agentService.getNameId(importService.nameData(file, fileType, request));
            if(ParameterUtil.isnull(agentMap)){
                fileMap.put("errormsg","代储点与代储库无关联，请查询后添加！");
                return fileMap;
            }

            if (list.size()>0) {
                errormsg = agentNumberService.importData(list, orgId, agentMap);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        fileMap.put("errormsg",errormsg);
        return fileMap;
    }

    @RequestMapping(value="/editCrkKcswData", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
	@ApiOperation(value="把出入库拿来的库存实物数据插入到代储库业务的库存实物表中", notes = "把出入库拿来的库存实物数据插入到代储库业务的库存实物表中")
	@ApiImplicitParams({
		@ApiImplicitParam(name="kcswData", value="出入库库存实物数据", paramType="form"),
		@ApiImplicitParam(name="orgId", value="当前单位ID", paramType="form"),
		@ApiImplicitParam(name="realName", value="当前操作人", paramType="form")
	})
	public Map<String, Object> editCrkKcswData(String kcswData, Integer orgId, String realName) {

		Map<String, Object> map = new HashMap<String, Object>();

		try {
			JSONArray json = JSONArray.fromObject(kcswData);
			if (json.size() > 0) {
				BusinessAgentNumber agentNumber = null;
				for (int i = 0; i < json.size(); i++) {

					JSONObject job = json.getJSONObject(i);
					Integer unitId = Integer.parseInt(job.get("unitid")+"");
					BusinessAgentDepot agentDepot = bgmxzService.selectAgentDepotObj(orgId, unitId);
					Integer agentDepotId = agentDepot.getId();
					Integer agentId = Integer.parseInt(agentDepot.getAgentId());
					Integer houseId = Integer.parseInt(job.get("ch")+"");
					Integer wareId = Integer.parseInt(job.get("hwh")+"");

					agentNumber = new BusinessAgentNumber();
					agentNumber.setOrgId(orgId);
					agentNumber.setAgentId(agentId);
					agentNumber.setAgentDepotId(agentDepotId);
					agentNumber.setStorehouseId(houseId);
					agentNumber.setWarehouseId(wareId);
					agentNumber.setLspz(Integer.parseInt(job.get("pz")+""));
					agentNumber.setNature(job.get("hwxz")+"");
					agentNumber.setLevel(Integer.parseInt(job.get("dj")+""));
					agentNumber.setHarvestYear(Integer.parseInt(job.get("scnf")+""));
                    if(ParameterUtil.isnull(job.get("scnf").toString())){
                        agentNumber.setHarvestYear(0);
                    }else{
                        agentNumber.setHarvestYear(Integer.parseInt(job.get("scnf")+""));
                    }
					agentNumber.setInventory(job.get("kcsl")+"");

					//检查这个仓房货位下的数据有没有，没有就新增，有就修改
					BusinessAgentNumberExample example = new BusinessAgentNumberExample();
					Criteria criteria = example.createCriteria();
					criteria.andOrgIdEqualTo(orgId);
					criteria.andAgentIdEqualTo(agentId);
					criteria.andAgentDepotIdEqualTo(agentDepotId);
					criteria.andStorehouseIdEqualTo(houseId);
					criteria.andWarehouseIdEqualTo(wareId);
					List<BusinessAgentNumber> list = agentNumberService.findByCondition(example);

					if (null != list && list.size() > 0) {
						agentNumber.setId(list.get(0).getId());
						agentNumberService.update(agentNumber);
					} else {
						//插入代储点的库存实物表
						agentNumber.setCreateTime(new Date());
						agentNumberService.add(agentNumber);
					}

				}
			}
			map.put("status", "200");
			map.put("msg", "操作成功！");
		} catch (NumberFormatException e) {
			map.put("msg", "操作失败！");
			e.printStackTrace();
		}

		return map;
	}
}
