package com.chinaitop.depot.agent.number.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.chinaitop.depot.agent.basic.mapper.BusinessAgentMapper;
import com.chinaitop.depot.agent.crk.mapper.BusinessAgentQcMapper;
import com.chinaitop.depot.agent.feign.AgileHouseFeignClient;
import com.chinaitop.depot.agent.number.mapper.BusinessAgentNumberMapper;
import com.chinaitop.depot.agent.number.model.BusinessAgentNumber;
import com.chinaitop.depot.agent.number.model.BusinessAgentNumberExample;
import com.chinaitop.depot.agent.number.service.agentNumberService;
import com.chinaitop.utils.ParameterUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class AgentNumberServiceImpl implements agentNumberService {

    @Resource
    private BusinessAgentNumberMapper businessAgentNumberMapper;

    @Resource
    private BusinessAgentMapper BusinessAgentMapper;

    @Resource
    private AgileHouseFeignClient agileHouseFeignClient;

    @Resource
    private BusinessAgentQcMapper businessAgentQcMapper;

    /**
     * 查询仓储处作业记录
     * * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getAgentNumberList(Map<String, Object> map) {
        //获取查询条件去出入库查数据
        List<Map<String, Object>> wareTimeList = businessAgentQcMapper.selectAgentHouseWareList(map);
        String str = JSON.toJSONString(wareTimeList);
        //调用depot-agile接口
        JSONArray crkobj =  agileHouseFeignClient.queryKcswData(str);
        //转换出入库查到的数据
        List<Map<String, Object>> listObjectFir = (List<Map<String, Object>>) JSONArray.parse(String.valueOf(crkobj));
        //查询代储点手动添加的数据
        List<Map<String, Object>> agentQc = businessAgentNumberMapper.getDataByExample(map);
        //两个list集合相加
        agentQc.addAll(listObjectFir);
        //返回集合
        return agentQc;
    }

    @Override
    public void add(BusinessAgentNumber agent) {
        businessAgentNumberMapper.insert(agent);
    }

    @Override
    public void update(BusinessAgentNumber camera) {
        businessAgentNumberMapper.updateByPrimaryKey(camera);
    }

    @Override
    public void updateDeleteById(Integer agentId) {
        businessAgentNumberMapper.deleteByPrimaryKey(agentId);
    }

    @Override
    public BusinessAgentNumber findById(Integer id) {
        return businessAgentNumberMapper.selectByPrimaryKey(id);
    }

    @Override
    public String importData(List<List<String>> list, Integer orgId, Map<String, Object> agentMap) {
        Date date = new Date();
        BusinessAgentNumber agent = null;
        Map<String,Object> enumData = new HashMap<>();

        String msg = "";
        int len = 0;

        for (int i = 0; i < list.size(); i++) {
            List<String> revList = list.get(i);

            for (int j = 0; j < revList.size(); j++) {
                switch(j) {
                    case 0:
                        //仓房名称
                        agent = new BusinessAgentNumber();
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 1 列的值的长度不能超过20！";
                            return msg;
                        }
                        enumData.put("storehouseName", revList.get(j));
                        enumData.put("depotId", agentMap.get("depotId"));
                        Map<String,Object> storehouseId = BusinessAgentMapper.getStorehouseId(enumData);
                        if(ParameterUtil.isnull(storehouseId)){
                            msg = "第 "+(i+3)+" 行第 1 列的值的不正确，请查询后再添加！";
                            return msg;
                        }
                        agent.setCreateTime(date);//创建时间
                        agent.setOrgId(orgId);
                        agent.setStorehouseId(Integer.parseInt(storehouseId.get("storehouseid").toString()));
                        break;
                    case 1:
                        //货位名称
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 2 列的值的长度不能超过20！";
                            return msg;
                        }
                        enumData.put("warehouseName", revList.get(j));
                        Map<String,Object> warehouseId = BusinessAgentMapper.getWarehouseId(enumData);
                        if(ParameterUtil.isnull(warehouseId)){
                            msg = "第 "+(i+3)+" 行第 2 列的值的不正确，请查询后再添加！";
                            return msg;
                        }
                        agent.setWarehouseId(Integer.parseInt(warehouseId.get("warehouseid").toString()));
                        break;
                    case 2:
                        //粮油品种
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 3 列的值的长度不能超过20！";
                            return msg;
                        }
                        enumData.put("enumName", revList.get(j));
                        enumData.put("parentId", "1061");
                        Map<String,Object> enumId = BusinessAgentMapper.getEnumId(enumData);
                        if(enumId.size() == 0){
                            msg = "第 "+(i+3)+" 行第 3 列的值的不正确，请查询后再添加！";
                            return msg;
                        }
                        agent.setLspz(Integer.parseInt(enumId.get("enumid").toString()));
                        break;
                    case 3:
                        //粮油性质
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 4 列的值的长度不能超过20！";
                            return msg;
                        }
                        if(ParameterUtil.isequal(revList.get(j),"其他粮")){
                            agent.setNature("3103");
                        }else{
                            enumData.put("enumName", revList.get(j));
                            enumData.put("parentId", "1032");
                            Map<String,Object> natureId = BusinessAgentMapper.getEnumChildId(enumData);

                            if(natureId.size() == 0){
                                msg = "第 "+(i+3)+" 行第 5 列的值的不正确，请查询后再添加！";
                                return msg;
                            }
                            agent.setNature(natureId.get("enumId").toString());
                        }
                        break;
                    case 4:
                        //等级
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 5 列的值的长度不能超过20！";
                            return msg;
                        }
                        enumData.put("enumName", revList.get(j));
                        enumData.put("parentId", "1002");
                        Map<String,Object> levelId = BusinessAgentMapper.getEnumId(enumData);
                        if(levelId.size() == 0){
                            msg = "第 "+(i+3)+" 行第 5 列的值的不正确，请查询后再添加！";
                            return msg;
                        }
                        agent.setLevel(Integer.parseInt(levelId.get("enumid").toString()));
                        break;
                    case 5:
                        //收获年度
                        len = revList.get(j).length();
                        if (len > 20) {
                            msg = "第 "+(i+3)+" 行第 6 列的值的长度不能超过20！";
                            return msg;
                        }
                        enumData.put("enumName", revList.get(j));
                        enumData.put("parentId", "1004");
                        Map<String,Object> yearId = BusinessAgentMapper.getEnumId(enumData);
                        if(yearId.size() == 0){
                            msg = "第 "+(i+3)+" 行第 6 列的值的不正确，请查询后再添加！";
                            return msg;
                        }
                        agent.setHarvestYear(Integer.parseInt(yearId.get("enumid").toString()));
                        break;
                    case 6:
                        //库存数量（公斤）
                        if (ParameterUtil.isnotnull(revList.get(j))) {
                            len = revList.get(j).length();
                            if (len > 50) {
                                msg = "第 "+(i+3)+" 行第 7 列的值的长度不能超过50！";
                                return msg;
                            }
                        }
                        agent.setInventory(revList.get(j));
                        break;
                    default :
                        break;
                }
            }
            agent.setAgentId(Integer.parseInt(agentMap.get("agentid").toString()));
            agent.setAgentDepotId(Integer.parseInt(agentMap.get("depotid").toString()));
            businessAgentNumberMapper.insert(agent);
        }
        return msg;
    }

	@Override
	public List<BusinessAgentNumber> findByCondition(BusinessAgentNumberExample example) {
		List<BusinessAgentNumber> list = businessAgentNumberMapper.selectByExample(example);
		return list;
	}
}
