package com.chinaitop.depot.agent.temperature.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.agent.basic.model.BusinessAgentTank;
import com.chinaitop.depot.agent.basic.service.agentService;
import com.chinaitop.depot.agent.temperature.model.BusinessAgentTemperature;
import com.chinaitop.depot.agent.temperature.model.TPoints;
import com.chinaitop.depot.agent.temperature.service.agentTemperatureService;
import com.chinaitop.depot.utils.ImportServiceImpl;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;

@RestController
@RequestMapping(value = "/agentTemperature")
@Api(description = "粮情数据")
public class AgentTemperatureController {

    @Resource
    private agentTemperatureService agentTemperatureService;

    @Resource
    private agentService agentService;

    @Resource
    private ImportServiceImpl importService;

    /**
     * 查询粮情信息记录
     *
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/getAgentTemperatureList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "粮情信息", notes = "查询粮情信息记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "agentId", value = "代储点id", paramType = "query"),
            @ApiImplicitParam(name = "agentDepotId", value = "代储库id", paramType = "query"),
            @ApiImplicitParam(name = "storehouseId", value = "仓房id", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "检测开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "检测结束时间", paramType = "query"),
            @ApiImplicitParam(name = "temperId", value = "粮情id", paramType = "query"),
            @ApiImplicitParam(name = "orgid", value = "组织id", paramType = "query")
    })
    public PageInfo<BusinessAgentTemperature> getAgentTemperatureList(Integer pageNum, Integer pageSize, Integer agentId, Integer agentDepotId,
                                                                 Integer storehouseId, String startTime, String endTime, String temperId, String orgid) {
        Map<String, Object> map = new HashMap<>();
        map.put("orgId", orgid);
        if (ParameterUtil.isnotnull(agentId)) {
            map.put("agentId", agentId);
        }
        if (ParameterUtil.isnotnull(agentDepotId)) {
            map.put("agentDepotId", agentDepotId);
        }
        if (ParameterUtil.isnotnull(storehouseId)) {
            map.put("storehouseId", storehouseId);
        }
        if (ParameterUtil.isnotnull(temperId)) {
            map.put("temperId", temperId);
        }
        if (ParameterUtil.isnotnull(startTime) && ParameterUtil.isnotnull(endTime)) {
            map.put("startTime", startTime + " 00:00:00");
            map.put("endTime", endTime + " 23:59:59");
        }

        PageHelper.startPage(pageNum, pageSize);
        List<BusinessAgentTemperature> list = agentTemperatureService.getAgentTemperatureList(map);
        PageInfo<BusinessAgentTemperature> pageInfo = new PageInfo<BusinessAgentTemperature>(list);
        return pageInfo;
    }

    /**
     * 保存粮情信息
     *
     * @param agentTemperatureJson 主键id
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "/saveAgent", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "保存粮情", notes = "保存粮情信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "agentTemperatureJson", value = "粮情信息", paramType = "form")
    })
    public Map<String, Object> saveAgent(String agentTemperatureJson, Integer orgId) throws Exception {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        BusinessAgentTemperature agent = (BusinessAgentTemperature) mapper.readValue(agentTemperatureJson, BusinessAgentTemperature.class);

        if (ParameterUtil.isnull(agent.getId())) {
            agent.setOrgId(orgId);
            agent.setCreateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//创建时间
            agentTemperatureService.add(agent);
        } else {
            agentTemperatureService.update(agent);
        }
        modelMap.put("agentId", agent.getId());
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 删除粮情信息
     *
     * @param id 主键id
     * @return
     */
    @RequestMapping(value = "/removeAgentTemperature", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "删除粮情信息", notes = "删除粮情信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "粮情主键id", paramType = "form")
    })
    public Map<String, Object> removeAgentTemperature(Integer id) {
        Map<String, Object> modelMap = new HashMap<>();
        if (id != null) {
            agentTemperatureService.updateDeleteById(id);
            modelMap.put("status", "success");
        }
        return modelMap;
    }

    /**
     * 根据id查找粮情信息
     *
     * @param id 主键id
     * @return
     */
    @RequestMapping(value = "/getAgentTemperatureEdit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据id查找粮情信息", notes = "根据id查找粮情信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "粮情主键id", paramType = "query")
    })
    public Map<String, Object> getAgentTemperatureEdit(Integer id) {
        Map<String, Object> modelMap = new HashMap<>();
        List<TPoints> tPointsList = new ArrayList<TPoints>();
        BusinessAgentTemperature agentTemper;
        if (id != null) {
            agentTemper = agentTemperatureService.findById(id);
            modelMap.put("agentTemper", agentTemper);
            String[] points = agentTemper.gettPoints().substring(2, agentTemper.gettPoints().length() - 2).split("\\},\\{");
            ObjectMapper mapper = new ObjectMapper();
            for (String point : points) {
                TPoints tPoints = JSON.parseObject(JSON.parse("{" + point + "}").toString(), TPoints.class);
                tPointsList.add(tPoints);
            }
            modelMap.put("tPoints", tPointsList);
        }
        return modelMap;
    }

    @RequestMapping(value = "/findTemperById", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据库房编码和测温时间查询检测记录包括层，行，各个节点的值", notes = "根据库房编码和测温时间查询检测记录包括层，行，各个节点的值")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "temperId", value = "主键id", paramType = "query"),
            @ApiImplicitParam(name = "sign", value = "区分手添和设备检测数据", paramType = "query")
    })
    public Map<String, ArrayList<Object>> findTemperById(String temperId, String sign) {
        Map<String, ArrayList<Object>> modelMap = new HashMap<>();
        ArrayList<Object> byIdDatas = null;
        if (StringUtils.isNotEmpty(temperId) && StringUtils.isNotBlank(sign)) {
            byIdDatas = agentTemperatureService.findTemperById(temperId, sign);
            modelMap.put("byIdDatas", byIdDatas);
        }
        return modelMap;
    }

    /**
     * 导入excel
     *
     * @param file     文件
     * @param fileType 类型：如xlsx
     * @param orgId    组织机构id
     * @return
     */
    @RequestMapping(value = "/importFile", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "导入excel", notes = "导入excel")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file", value = "文件数据", paramType = "query"),
            @ApiImplicitParam(name = "fileType", value = "文件名", paramType = "query"),
            @ApiImplicitParam(name = "rootPath", value = "路径", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织机构id", paramType = "query")
    })
    public Map<String, Object> importFile(MultipartFile file, String fileType, HttpServletRequest request, Integer orgId) {

        String[] titleArr = {"测温线缆列号(X)", "测温线缆行号(Y)", "测温点高度(Z)", "温度值"};//第一行的title
        boolean[] emptyArr = {false, false, false, false};//指定列可为空，不可为空值为false.

        List<List<String>> list = new ArrayList<List<String>>();//该list为数据返回的list
        Map<String, Object> fileMap = new HashMap<>();
        String errormsg = null;
        try {
            errormsg = importService.importFile(file, fileType, request, titleArr, emptyArr, list, 10);

            if (errormsg != null && !"".equals(errormsg)) {
                fileMap.put("errormsg", errormsg);
                return fileMap;
            }

            //查询代储点名称和代储库名称对应的id
            Map<String, Object> agentMap = importService.temperData(file, fileType, request);

            if (list.size() > 0) {
                errormsg = agentTemperatureService.importData(list, orgId, agentMap);

                System.out.println(list);
                System.out.println(errormsg);
                //errormsg = "导入成功！";
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        fileMap.put("errormsg", errormsg);
        return fileMap;
    }

    /**
     * 根据登录人机构id查找代储库单位下的所有仓房信息
     *
     * @param orgId 单位ID
     * @return map
     */
    @RequestMapping(value = "/getAgentStoreInfo", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据登录人机构id查找代储库单位下的仓房信息", notes = "根据登录人机构id查找代储库单位下的仓房信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query")
    })
    public Map<String, Object> getAgentStoreInfo(Integer orgId) {
        Map<String, Object> map = agentTemperatureService.getAgentStoreInfoMap(orgId);
        //处理驼峰命名大写问题
        Map<String, Object> mapData = ParameterUtil.slashCapitals(map);
        return mapData;
    }

}
