package com.chinaitop.depot.agent.temperature.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.agent.basic.mapper.BusinessAgentMapper;
import com.chinaitop.depot.agent.temperature.mapper.BusinessAgentTemperatureMapper;
import com.chinaitop.depot.agent.temperature.model.BusinessAgentTemperature;
import com.chinaitop.depot.agent.temperature.model.BusinessAgentTemperatureExample;
import com.chinaitop.depot.agent.temperature.model.TPoints;
import com.chinaitop.depot.agent.temperature.service.agentTemperatureService;
import com.chinaitop.utils.ParameterUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class AgentTemperatureServiceImpl implements agentTemperatureService {

    @Resource
    private BusinessAgentTemperatureMapper businessAgentTemperatureMapper;

    @Resource
    private BusinessAgentMapper BusinessAgentMapper;

    /**
     * 查询仓储处作业记录
     * * @param map
     * @return
     */
    @Override
    public List<BusinessAgentTemperature> getAgentTemperatureList(Map<String, Object> map) {
        return businessAgentTemperatureMapper.getDataByExample(map);
    }

    @Override
    public void add(BusinessAgentTemperature agent) {
        businessAgentTemperatureMapper.insert(agent);
    }

    @Override
    public void update(BusinessAgentTemperature camera) {
        businessAgentTemperatureMapper.updateByPrimaryKey(camera);
    }

    @Override
    public void updateDeleteById(Integer agentId) {
        businessAgentTemperatureMapper.deleteByPrimaryKey(agentId);
    }

    @Override
    public BusinessAgentTemperature findById(Integer id) {
        return businessAgentTemperatureMapper.selectByPrimaryKey(id);
    }

    @Override
    public ArrayList<Object> findTemperById(String temperId, String sign) {
        // 区分手添和设备检测数据
        String s = null;
        if ("0".equals(sign)) { // 智能仓房设备检测数据
            s = businessAgentTemperatureMapper.getTestDataById(temperId); //查询智能仓房设备检测数据
        } else {
            BusinessAgentTemperature byId = businessAgentTemperatureMapper.getById(temperId).get(0);
            s = byId.gettPoints();
        }
        ArrayList<Object> tP = ParameterUtil.jsonToObjects(s, TPoints.class);
        ArrayList<TPoints> tPoints = Lists.newArrayList();
        tP.forEach(t -> {
            tPoints.add((TPoints)t);
        });
        ArrayList<Object> objects = Lists.newArrayList();
        HashMap<Object, Object> layerDataMap = Maps.newLinkedHashMap();//记录每层最高温，最低温和平均温度
        if (tPoints.size() > 0) {
            float rowNumbers = 0;//记录层数
            Map<Integer, Map<Integer, Map<Integer, List<TPoints>>>> collect1 = tPoints.stream().collect(Collectors.groupingBy(t -> Integer.parseInt(t.getY()), Collectors.groupingBy(t -> Integer.parseInt(t.getZ()), Collectors.groupingBy(t -> Integer.parseInt(t.getX())))));
            for (Map.Entry<Integer, Map<Integer, Map<Integer, List<TPoints>>>> entryG : collect1.entrySet()) {
                float layerMax = 0,layerMin = 99,layerAvg = 0,numbers = 0;
                rowNumbers++;
                //每层数据
                Map<Integer, Map<Integer, List<TPoints>>> layerGroupV = entryG.getValue();
                for (Map.Entry<Integer, Map<Integer, List<TPoints>>> rows : layerGroupV.entrySet()) {
                    HashMap<Object, Object> objectObjectHashMap = Maps.newLinkedHashMap();

                    //每一行数据
                    Map<Integer, List<TPoints>> value = rows.getValue();
                    int a = 1;
                    objectObjectHashMap.put(a,entryG.getKey() + "-" + rows.getKey());
                    for (Map.Entry<Integer, List<TPoints>> orws : value.entrySet()) {
                        //列数据
                        List<TPoints> column = orws.getValue();
                        TPoints t = column.get(0);
                        objectObjectHashMap.put(++a,t.getTemp());

                        if(!ParameterUtil.isequal(250.0f,t.getTemp().floatValue())){//不是设备异常才记录
                            if(t.getTemp().floatValue() > layerMax)layerMax = t.getTemp().floatValue();//获取层最高温度
                            if(t.getTemp().floatValue() < layerMin)layerMin = t.getTemp().floatValue();//获取层最低温度
                            layerAvg += t.getTemp().floatValue();//获取层平均温度
                            numbers++;
                        }
                    }
                    objects.add(objectObjectHashMap);
                }
                layerDataMap.put(entryG.getKey(),String.format("%.1f",layerAvg/numbers)+","+layerMax+","+layerMin);
            }
            layerDataMap.put("rowNumbers",rowNumbers);
        }
        objects.add(layerDataMap);
        return objects;
    }

    @Override
    public String importData(List<List<String>> list, Integer orgId, Map<String, Object> agentMap) {
        String msg = "";

        BusinessAgentTemperature agentTemper = new BusinessAgentTemperature();;
        Map<String,Object> enumData = new HashMap<>();

        Map<String, Object> agentsMap = BusinessAgentMapper.getNameId(agentMap);;
        if(ParameterUtil.isnull(agentsMap) || ParameterUtil.isnull(agentsMap.get("agentId"))){
            msg = "代储点信息查询不到，请确认后再导入！";
            return msg;
        }
        agentTemper.setAgentId(Integer.parseInt(agentsMap.get("agentid").toString()));
        if(ParameterUtil.isnull(agentsMap.get("depotid"))){
            msg = "代储库信息查询不到，请确认后再导入！";
            return msg;
        }
        agentTemper.setAgentDepotId(Integer.parseInt(agentsMap.get("depotid").toString()));
        agentTemper.setWeather(agentMap.get("weather").toString());

        //仓房名称
        enumData.put("storehouseName", agentMap.get("storehouseName"));
        enumData.put("depotId", agentsMap.get("depotId"));
        Map<String,Object> storehouseId = BusinessAgentMapper.getStorehouseId(enumData);
        if(ParameterUtil.isnull(storehouseId)){
            msg = "仓房信息查询不到，请确认后再导入！";
            return msg;
        }
        agentTemper.setStorehouseId(Integer.parseInt(storehouseId.get("storehouseid").toString()));

        agentTemper.setActualAmount(agentMap.get("actualAmount").toString());

        //粮油品种
        enumData.put("enumName", agentMap.get("lspz"));
        enumData.put("parentId", "1061");
        Map<String,Object> enumId = BusinessAgentMapper.getEnumId(enumData);
        if(ParameterUtil.isnull(enumId.get("enumid"))){
            msg = "粮油品种信息查询不到，请确认后再导入！";
            return msg;
        }
        agentTemper.setLspz(Integer.parseInt(enumId.get("enumid").toString()));

        //收获年度
        enumData.put("enumName", agentMap.get("harvestYear").toString().substring(0,4));
        enumData.put("parentId", "1004");
        Map<String,Object> yearId = BusinessAgentMapper.getEnumId(enumData);
        if(ParameterUtil.isnull(yearId.get("enumid"))){
            msg = "收获年度信息查询不到，请确认后再导入！";
            return msg;
        }
        agentTemper.setHarvestYear(Integer.parseInt(yearId.get("enumid").toString()));

        agentTemper.setEnterTime(ParameterUtil.string2date(agentMap.get("enterTime").toString()));

        //等级
        enumData.put("enumName", agentMap.get("level"));
        enumData.put("parentId", "1002");
        Map<String,Object> levelId = BusinessAgentMapper.getEnumId(enumData);
        if(ParameterUtil.isnull(levelId.get("enumid"))){
            msg = "等级信息查询不到，请确认后再导入！";
            return msg;
        }
        agentTemper.setLevel(Integer.parseInt(levelId.get("enumid").toString()));

        //粮油性质
        if(ParameterUtil.isequal(agentMap.get("nature").toString(),"其他粮")){
            agentTemper.setNature("3103");
        }else{
            enumData.put("enumName", agentMap.get("nature").toString());
            enumData.put("parentId", "1032");
            Map<String,Object> natureId = BusinessAgentMapper.getEnumChildId(enumData);
            if(ParameterUtil.isnull(natureId.get("enumid"))){
                msg = "粮油性质信息查询不到，请确认后再导入！";
                return msg;
            }
            agentTemper.setNature(natureId.get("enumid").toString());
        }

        agentTemper.setKeeper(agentMap.get("keeper").toString());
        agentTemper.setHouseTemperature(new BigDecimal(agentMap.get("houseTemperature").toString()));
        agentTemper.setHouseHumidity(new BigDecimal(agentMap.get("houseHumidity").toString()));
        agentTemper.setGasTemperature(new BigDecimal(agentMap.get("gasTemperature").toString()));
        agentTemper.setGasHumidity(new BigDecimal(agentMap.get("gasHumidity").toString()));
        agentTemper.setDetectionTime(ParameterUtil.string2date(agentMap.get("detectionTime").toString()));
        agentTemper.settMax(new BigDecimal(agentMap.get("t_max").toString()));
        agentTemper.settMin(new BigDecimal(agentMap.get("t_min").toString()));
        agentTemper.settAvg(new BigDecimal(agentMap.get("t_avg").toString()));
        agentTemper.setOrgId(orgId);

        Map<String,Object> pointMap = new HashMap<>();//一行温度数据
        List<String> pointList = new ArrayList<String>();//温度集合数据

        for (int i = 0; i < list.size(); i++) {
            List<String> revList = list.get(i);
            for (int j = 0; j < revList.size(); j++) {
                switch(j) {
                    case 0:
                        //测温线缆列号(X)
                        pointMap = new HashMap<>();
                        pointMap.put("x",revList.get(j));
                        break;
                    case 1:
                        //测温线缆行号(Y)
                        pointMap.put("y",revList.get(j));
                        break;
                    case 2:
                        //测温点高度(Z)
                        pointMap.put("z",revList.get(j));
                        break;
                    case 3:
                        //温度值
                        pointMap.put("temp",revList.get(j));
                        break;
                    default :
                        break;
                }
            }
            pointList.add(JSONObject.toJSONString(pointMap));
        }

        agentTemper.settPoints(pointList.toString().replace(" ",""));
        businessAgentTemperatureMapper.insert(agentTemper);

        return msg;
    }

    /**
     * 根据登录人机构id查找代储库单位下的所有仓房信息
     *
     * @param orgId 单位ID
     * @return map
     */
    @Override
    public Map<String, Object> getAgentStoreInfoMap(Integer orgId) {
        List<Map<String, Object>> listMap = businessAgentTemperatureMapper.getAgentStoreInfoMap(orgId);
        Map<String, Object> objMap = new HashMap<>();
        listMap.forEach(map -> {
            String keys = "";
            for (String key : map.keySet()) {
                if (key.equals("storehouse_code")) {
                    keys = map.get(key).toString();
                }
            }
            objMap.put(keys, map);
        });
        Map<String, Object> reMap = new HashMap<>();
        reMap.put("agentStoreCode", objMap);
        return reMap;
    }

}
